@@ -13,11 +13,11 @@ namespace fff
 {
 struct AfsConfig
 {
-    Zstring resourceDirPathPf; //directory to read AFS-specific files
-    Zstring configDirPathPf;   //directory to store AFS-specific files
+    Zstring resourceDirPath; //directory to read AFS-specific files
+    Zstring configDirPath;   //directory to store AFS-specific files
 };
 void initAfs(const AfsConfig& cfg);
-void teardownAfs();
+[[nodiscard]] std::wstring /*warningMsg*/ teardownAfs();
 
 AbstractPath getNullPath();
 AbstractPath createAbstractPath(const Zstring& itemPathPhrase); //noexcept
