@@ -1,4 +1,4 @@
-// *****************************************************************************
+﻿// *****************************************************************************
 // * This file is part of the FreeFileSync project. It is distributed under    *
 // * GNU General Public License: https://www.gnu.org/licenses/gpl-3.0          *
 // * Copyright (C) Zenju (zenju AT freefilesync DOT org) - All Rights Reserved *
@@ -72,7 +72,7 @@ struct ResolvedBaseFolders
 
 
 ResolvedBaseFolders initializeBaseFolders(const std::vector<FolderPairCfg>& fpCfgList,
-                                          bool allowUserInteraction,
+                                          const AFS::RequestPasswordFun& requestPassword /*throw X*/,
                                           WarningDialogs& warnings,
                                           PhaseCallback& callback /*throw X*/) //throw X
 {
@@ -95,8 +95,8 @@ ResolvedBaseFolders initializeBaseFolders(const std::vector<FolderPairCfg>& fpCf
             allFolders.insert(output.resolvedPairs.back().folderPathRight);
         }
         //---------------------------------------------------------------------------
-        output.baseFolderStatus = getFolderStatusNonBlocking(allFolders,
-                                                             allowUserInteraction, callback); //throw X
+        output.baseFolderStatus = getFolderStatusParallel(allFolders,
+                                                          true /*authenticateAccess*/, requestPassword, callback); //throw X
         if (!output.baseFolderStatus.failedChecks.empty())
         {
             std::wstring msg = _("Cannot find the following folders:") + L'\n';
@@ -129,8 +129,8 @@ ResolvedBaseFolders initializeBaseFolders(const std::vector<FolderPairCfg>& fpCf
     //---------------------------------------------------------------------------
     std::map<std::pair<AfsDevice, ZstringNoCase>, std::set<AbstractPath>> ciPathAliases;
 
-    for (const AbstractPath& ap : allFolders)
-        ciPathAliases[std::pair(ap.afsDevice, ap.afsPath.value)].insert(ap);
+    for (const AbstractPath& folderPath : allFolders)
+        ciPathAliases[std::pair(folderPath.afsDevice, folderPath.afsPath.value)].insert(folderPath);
 
     if (std::any_of(ciPathAliases.begin(), ciPathAliases.end(), [](const auto& item) { return item.second/*aliases*/.size() > 1; }))
     {
@@ -144,6 +144,8 @@ ResolvedBaseFolders initializeBaseFolders(const std::vector<FolderPairCfg>& fpCf
             }
 
         callback.reportWarning(msg, warnings.warnFoldersDifferInCase); //throw X
+
+        //what about /folder and /Folder/subfolder? => yes, inconsistent, but doesn't matter for FFS
     }
     //---------------------------------------------------------------------------
 
@@ -163,7 +165,7 @@ public:
     //create comparison result table and fill category except for files existing on both sides: undefinedFiles and undefinedSymlinks are appended!
     std::shared_ptr<BaseFolderPair> compareByTimeSize(const ResolvedFolderPair& fp, const FolderPairCfg& fpConfig) const;
     std::shared_ptr<BaseFolderPair> compareBySize    (const ResolvedFolderPair& fp, const FolderPairCfg& fpConfig) const;
-    std::list<std::shared_ptr<BaseFolderPair>> compareByContent(const std::vector<std::pair<ResolvedFolderPair, FolderPairCfg>>& workLoad) const;
+    std::vector<std::shared_ptr<BaseFolderPair>> compareByContent(const std::vector<std::pair<ResolvedFolderPair, FolderPairCfg>>& workLoad) const;
 
 private:
     ComparisonBuffer           (const ComparisonBuffer&) = delete;
@@ -213,9 +215,10 @@ ComparisonBuffer::ComparisonBuffer(const std::set<DirectoryKey>& folderKeys,
 
     const int64_t totalTimeSec = std::chrono::duration_cast<std::chrono::seconds>(std::chrono::steady_clock::now() - compareStartTime).count();
 
-    callback.logInfo(_("Comparison finished:") + L' ' +
-                     _P("1 item found", "%x items found", itemsReported) + L" | " +
-                     _("Time elapsed:") + L' ' + copyStringTo<std::wstring>(wxTimeSpan::Seconds(totalTimeSec).Format())); //throw X
+    callback.logMessage(_("Comparison finished:") + L' ' +
+                        _P("1 item found", "%x items found", itemsReported) + SPACED_DASH +
+                        _("Time elapsed:") + L' ' + copyStringTo<std::wstring>(wxTimeSpan::Seconds(totalTimeSec).Format()),
+                        PhaseCallback::MsgType::info); //throw X
     //------------------------------------------------------------------
 
     //folderStatus_.existing already in buffer, now create entries for the rest:
@@ -248,15 +251,15 @@ template <SelectSide side, class FileOrLinkPair> inline
 Zstringc getConflictInvalidDate(const FileOrLinkPair& file)
 {
     return utfTo<Zstringc>(replaceCpy(_("File %x has an invalid date."), L"%x", fmtPath(AFS::getDisplayPath(file.template getAbstractPath<side>()))) + L'\n' +
-                           _("Date:") + L' ' + formatUtcToLocalTime(file.template getLastWriteTime<side>()));
+                           TAB_SPACE + _("Date:") + L' ' + formatUtcToLocalTime(file.template getLastWriteTime<side>()));
 }
 
 
 Zstringc getConflictSameDateDiffSize(const FilePair& file)
 {
     return utfTo<Zstringc>(_("Files have the same date but a different size.") + L'\n' +
-                           arrowLeft  + L' ' + _("Date:") + L' ' + formatUtcToLocalTime(file.getLastWriteTime<SelectSide::left >()) + L"    " + _("Size:") + L' ' + formatNumber(file.getFileSize<SelectSide::left>()) + L'\n' +
-                           arrowRight + L' ' + _("Date:") + L' ' + formatUtcToLocalTime(file.getLastWriteTime<SelectSide::right>()) + L"    " + _("Size:") + L' ' + formatNumber(file.getFileSize<SelectSide::right>()));
+                           TAB_SPACE + arrowLeft  + L' ' + _("Date:") + L' ' + formatUtcToLocalTime(file.getLastWriteTime<SelectSide::left >()) + TAB_SPACE + _("Size:") + L' ' + formatNumber(file.getFileSize<SelectSide::left>()) + L'\n' +
+                           TAB_SPACE + arrowRight + L' ' + _("Date:") + L' ' + formatUtcToLocalTime(file.getLastWriteTime<SelectSide::right>()) + TAB_SPACE + _("Size:") + L' ' + formatNumber(file.getFileSize<SelectSide::right>()));
 }
 
 
@@ -269,8 +272,8 @@ Zstringc getConflictSkippedBinaryComparison()
 Zstringc getDescrDiffMetaShortnameCase(const FileSystemObject& fsObj)
 {
     return utfTo<Zstringc>(_("Items differ in attributes only") + L'\n' +
-                           arrowLeft  + L' ' + fmtPath(fsObj.getItemName<SelectSide::left >()) + L'\n' +
-                           arrowRight + L' ' + fmtPath(fsObj.getItemName<SelectSide::right>()));
+                           TAB_SPACE + arrowLeft  + L' ' + fmtPath(fsObj.getItemName<SelectSide::left >()) + L'\n' +
+                           TAB_SPACE + arrowRight + L' ' + fmtPath(fsObj.getItemName<SelectSide::right>()));
 }
 
 
@@ -279,8 +282,8 @@ template <class FileOrLinkPair>
 Zstringc getDescrDiffMetaData(const FileOrLinkPair& file)
 {
     return utfTo<Zstringc>(_("Items differ in attributes only") + L'\n' +
-                           arrowLeft  + L' ' + _("Date:") + L' ' + formatUtcToLocalTime(file.template getLastWriteTime<SelectSide::left >()) + L'\n' +
-                           arrowRight + L' ' + _("Date:") + L' ' + formatUtcToLocalTime(file.template getLastWriteTime<SelectSide::right>()));
+                           TAB_SPACE + arrowLeft  + L' ' + _("Date:") + L' ' + formatUtcToLocalTime(file.template getLastWriteTime<SelectSide::left >()) + L'\n' +
+                           TAB_SPACE + arrowRight + L' ' + _("Date:") + L' ' + formatUtcToLocalTime(file.template getLastWriteTime<SelectSide::right>()));
 }
 #endif
 
@@ -476,18 +479,21 @@ namespace
 {
 void categorizeFileByContent(FilePair& file, const std::wstring& txtComparingContentOfFiles, AsyncCallback& acb, std::mutex& singleThread) //throw ThreadStopRequest
 {
-    acb.updateStatus(replaceCpy(txtComparingContentOfFiles, L"%x", fmtPath(file.getRelativePathAny()))); //throw ThreadStopRequest
-
     bool haveSameContent = false;
     const std::wstring errMsg = tryReportingError([&]
     {
-        AsyncItemStatReporter statReporter(1, file.getFileSize<SelectSide::left>(), acb);
+        std::wstring statusMsg = replaceCpy(txtComparingContentOfFiles, L"%x", fmtPath(file.getRelativePathAny()));
 
-        //callbacks run *outside* singleThread_ lock! => fine
-        auto notifyUnbufferedIO = [&statReporter](int64_t bytesDelta)
+        ItemStatReporter statReporter(1, file.getFileSize<SelectSide::left>(), acb);
+        PercentStatReporter percentReporter(statusMsg, file.getFileSize<SelectSide::left>(), statReporter);
+
+        acb.updateStatus(std::move(statusMsg)); //throw ThreadStopRequest
+
+        //callbacks run *outside* singleThread lock! => fine
+        auto notifyUnbufferedIO = [&percentReporter](int64_t bytesDelta)
         {
-            statReporter.reportDelta(0, bytesDelta);
-            interruptionPoint(); //throw ThreadStopRequest
+            percentReporter.updateDeltaAndStatus(bytesDelta); //throw ThreadStopRequest
+            interruptionPoint(); //throw ThreadStopRequest => not reliably covered by PercentStatReporter::updateDeltaAndStatus()!
         };
 
         haveSameContent = parallel::filesHaveSameContent(file.getAbstractPath<SelectSide::left >(),
@@ -523,7 +529,7 @@ void categorizeFileByContent(FilePair& file, const std::wstring& txtComparingCon
 }
 
 
-std::list<std::shared_ptr<BaseFolderPair>> ComparisonBuffer::compareByContent(const std::vector<std::pair<ResolvedFolderPair, FolderPairCfg>>& workLoad) const
+std::vector<std::shared_ptr<BaseFolderPair>> ComparisonBuffer::compareByContent(const std::vector<std::pair<ResolvedFolderPair, FolderPairCfg>>& workLoad) const
 {
     struct ParallelOps
     {
@@ -546,8 +552,7 @@ std::list<std::shared_ptr<BaseFolderPair>> ComparisonBuffer::compareByContent(co
         fpWorkload.push_back({posL, posR, std::move(filesToCompareBytewise)});
     };
 
-    //PERF_START;
-    std::list<std::shared_ptr<BaseFolderPair>> output;
+    std::vector<std::shared_ptr<BaseFolderPair>> output;
 
     const Zstringc txtConflictSkippedBinaryComparison = getConflictSkippedBinaryComparison(); //avoid premature pess.: save memory via ref-counted string
 
@@ -603,11 +608,10 @@ std::list<std::shared_ptr<BaseFolderPair>> ComparisonBuffer::compareByContent(co
 
         AsyncCallback acb;                       //
         std::function<void()> scheduleMoreTasks; //manage life time: enclose ThreadGroup!
-        const std::wstring txtComparingContentOfFiles = _("Comparing content of files %x"); //
 
         ThreadGroup<std::function<void()>> tg(std::numeric_limits<size_t>::max(), Zstr("Binary Comparison"));
 
-        scheduleMoreTasks = [&]
+        scheduleMoreTasks = [&, txtComparingContentOfFiles = _("Comparing content of files %x")]
         {
             bool wereDone = true;
 
@@ -662,7 +666,7 @@ std::list<std::shared_ptr<BaseFolderPair>> ComparisonBuffer::compareByContent(co
 class MergeSides
 {
 public:
-    MergeSides(const std::map<ZstringNoCase, Zstringc>& errorsByRelPath,
+    MergeSides(const std::unordered_map<ZstringNoCase, Zstringc>& errorsByRelPath,
                std::vector<FilePair*>& undefinedFilesOut,
                std::vector<SymlinkPair*>& undefinedSymlinksOut) :
         errorsByRelPath_(errorsByRelPath),
@@ -686,8 +690,8 @@ private:
 
     const Zstringc* checkFailedRead(FileSystemObject& fsObj, const Zstringc* errorMsg);
 
-    const std::map<ZstringNoCase, Zstringc>& errorsByRelPath_; //base-relative paths or empty if read-error for whole base directory
-    std::vector<FilePair*>&    undefinedFiles_;
+    const std::unordered_map<ZstringNoCase, Zstringc>& errorsByRelPath_; //base-relative paths or empty if read-error for whole base directory
+    std::vector<FilePair*>& undefinedFiles_;
     std::vector<SymlinkPair*>& undefinedSymlinks_;
 };
 
@@ -696,9 +700,10 @@ inline
 const Zstringc* MergeSides::checkFailedRead(FileSystemObject& fsObj, const Zstringc* errorMsg)
 {
     if (!errorMsg)
-        if (const auto it = errorsByRelPath_.find(fsObj.getRelativePathAny());
-            it != errorsByRelPath_.end())
-            errorMsg = &it->second;
+        if (!errorsByRelPath_.empty()) //only pay for ZstringNoCase construction when needed
+            if (const auto it = errorsByRelPath_.find(fsObj.getRelativePathAny());
+                it != errorsByRelPath_.end())
+                errorMsg = &it->second;
 
     if (errorMsg)
     {
@@ -710,27 +715,47 @@ const Zstringc* MergeSides::checkFailedRead(FileSystemObject& fsObj, const Zstri
 }
 
 
+template <class MapType, class Function>
+void forEachSorted(const MapType& fileMap, Function fun)
+{
+    using FileRef = const typename MapType::value_type*;
+
+    std::vector<FileRef> fileList;
+    fileList.reserve(fileMap.size());
+
+    for (const auto& item : fileMap)
+        fileList.push_back(&item);
+
+    //sort for natural default sequence on UI file grid:
+    std::sort(fileList.begin(), fileList.end(), [](const FileRef& lhs, const FileRef& rhs) { return compareNoCase(lhs->first /*item name*/, rhs->first) < 0; });
+
+    for (const auto& item : fileList)
+        fun(item->first, item->second);
+}
+
+
 template <SelectSide side>
 void MergeSides::fillOneSide(const FolderContainer& folderCont, const Zstringc* errorMsg, ContainerObject& output)
 {
-    for (const auto& [fileName, attrib] : folderCont.files)
+    forEachSorted(folderCont.files, [&](const Zstring& fileName, const FileAttributes& attrib)
     {
         FilePair& newItem = output.addSubFile<side>(fileName, attrib);
         checkFailedRead(newItem, errorMsg);
-    }
+    });
 
-    for (const auto& [linkName, attrib] : folderCont.symlinks)
+    forEachSorted(folderCont.symlinks, [&](const Zstring& linkName, const LinkAttributes& attrib)
     {
         SymlinkPair& newItem = output.addSubLink<side>(linkName, attrib);
         checkFailedRead(newItem, errorMsg);
-    }
+    });
 
-    for (const auto& [folderName, attrAndSub] : folderCont.folders)
+    forEachSorted(folderCont.folders, [&](const Zstring& folderName, const std::pair<FolderAttributes, FolderContainer>& attrib)
     {
-        FolderPair& newFolder = output.addSubFolder<side>(folderName, attrAndSub.first);
+        FolderPair& newFolder = output.addSubFolder<side>(folderName, attrib.first);
         const Zstringc* errorMsgNew = checkFailedRead(newFolder, errorMsg);
-        fillOneSide<side>(attrAndSub.second, errorMsgNew, newFolder); //recurse
-    }
+
+        fillOneSide<side>(attrib.second, errorMsgNew, newFolder); //recurse
+    });
 }
 
 
@@ -739,28 +764,29 @@ void matchFolders(const MapType& mapLeft, const MapType& mapRight, ProcessLeftOn
 {
     struct FileRef
     {
-        Zstring upperCaseName; //buffer expensive getUpperCase() calls!!
+        //perf: buffer ZstringNoCase instead of compareNoCase()/equalNoCase()? => makes no (significant) difference!
         const typename MapType::value_type* ref;
-        bool leftSide;
+        SelectSide side;
     };
     std::vector<FileRef> fileList;
     fileList.reserve(mapLeft.size() + mapRight.size()); //perf: ~5% shorter runtime
 
-    for (const auto& item : mapLeft ) fileList.push_back({getUpperCase(item.first), &item, true});
-    for (const auto& item : mapRight) fileList.push_back({getUpperCase(item.first), &item, false});
+    for (const auto& item : mapLeft ) fileList.push_back({&item, SelectSide::left});
+    for (const auto& item : mapRight) fileList.push_back({&item, SelectSide::right});
 
-    //primary sort: ignore unicode normal form and case
-    //bonus: natural default sequence on file guid UI
-    std::sort(fileList.begin(), fileList.end(), [](const FileRef& lhs, const FileRef& rhs) { return lhs.upperCaseName < rhs.upperCaseName; });
+    //primary sort: ignore Unicode normal form and upper/lower case
+    //bonus: natural default sequence on UI file grid
+    std::sort(fileList.begin(), fileList.end(), [](const FileRef& lhs, const FileRef& rhs) { return compareNoCase(lhs.ref->first /*item name*/, rhs.ref->first) < 0; });
 
-    auto tryMatchRange = [&](auto it, auto itLast)
+    using ItType = typename std::vector<FileRef>::iterator;
+    auto tryMatchRange = [&](ItType it, ItType itLast) //auto parameters? compiler error on VS 17.2...
     {
-        const size_t equalCountL = std::count_if(it, itLast, [](const FileRef& fr) { return fr.leftSide; });
+        const size_t equalCountL = std::count_if(it, itLast, [](const FileRef& fr) { return fr.side == SelectSide::left; });
         const size_t equalCountR = itLast - it - equalCountL;
 
         if (equalCountL == 1 && equalCountR == 1) //we have a match
         {
-            if (it->leftSide)
+            if (it->side == SelectSide::left)
                 bo(*it[0].ref, *it[1].ref);
             else
                 bo(*it[1].ref, *it[0].ref);
@@ -777,7 +803,7 @@ void matchFolders(const MapType& mapLeft, const MapType& mapRight, ProcessLeftOn
     for (auto it = fileList.begin(); it != fileList.end();)
     {
         //find equal range: ignore case, ignore Unicode normalization
-        auto itEndEq = std::find_if(it + 1, fileList.end(), [&](const FileRef& fr) { return fr.upperCaseName != it->upperCaseName; });
+        auto itEndEq = std::find_if(it + 1, fileList.end(), [&](const FileRef& fr) { return !equalNoCase(fr.ref->first, it->ref->first); });
         if (!tryMatchRange(it, itEndEq))
         {
             //secondary sort: respect case, ignore unicode normal forms
@@ -792,7 +818,7 @@ void matchFolders(const MapType& mapLeft, const MapType& mapRight, ProcessLeftOn
                     const Zstringc& conflictMsg = getConflictAmbiguousItemName(itCase->ref->first);
                     std::for_each(itCase, itEndCase, [&](const FileRef& fr)
                     {
-                        if (fr.leftSide)
+                        if (fr.side == SelectSide::left)
                             lo(*fr.ref, &conflictMsg);
                         else
                             ro(*fr.ref, &conflictMsg);
@@ -922,7 +948,7 @@ std::shared_ptr<BaseFolderPair> ComparisonBuffer::performComparison(const Resolv
     cb_.updateStatus(_("Generating file list...")); //throw X
     cb_.requestUiUpdate(true /*force*/); //throw X
 
-    std::map<ZstringNoCase, Zstringc> failedReads; //base-relative paths or empty if read-error for whole base directory
+    std::unordered_map<ZstringNoCase, Zstringc> failedReads; //base-relative paths or empty if read-error for whole base directory
 
     auto evalFolderContent = [&](const AbstractPath& folderPath) -> const FolderContainer&
     {
@@ -932,7 +958,7 @@ std::shared_ptr<BaseFolderPair> ComparisonBuffer::performComparison(const Resolv
         //mix failedFolderReads with failedItemReads:
         //associate folder traversing errors with folder (instead of child items only) to show on GUI! See "MergeSides"
         //=> minor pessimization for "excludefilterFailedRead" which needlessly excludes parent folders, too
-        auto append = [&](const std::map<Zstring, Zstringc>& c)
+        auto append = [&](const std::unordered_map<Zstring, Zstringc>& c)
         {
             for (const auto& [relPath, errorMsg] : c)
                 failedReads.emplace(relPath, errorMsg);
@@ -1002,12 +1028,12 @@ std::shared_ptr<BaseFolderPair> ComparisonBuffer::performComparison(const Resolv
 
 FolderComparison fff::compare(WarningDialogs& warnings,
                               int fileTimeTolerance,
-                              bool allowUserInteraction,
+                              const AFS::RequestPasswordFun& requestPassword /*throw X*/,
                               bool runWithBackgroundPriority,
                               bool createDirLocks,
                               std::unique_ptr<LockHolder>& dirLocks,
                               const std::vector<FolderPairCfg>& fpCfgList,
-                              ProcessCallback& callback)
+                              ProcessCallback& callback /*throw X*/) //throw X
 {
     //PERF_START;
 
@@ -1034,11 +1060,11 @@ FolderComparison fff::compare(WarningDialogs& warnings,
     }
     catch (const FileError& e) //failure is not critical => log only
     {
-        callback.logInfo(e.toString()); //throw X
+        callback.logMessage(e.toString(), PhaseCallback::MsgType::warning); //throw X
     }
 
     const ResolvedBaseFolders& resInfo = initializeBaseFolders(fpCfgList,
-                                                               allowUserInteraction, warnings, callback); //throw X
+                                                               requestPassword, warnings, callback); //throw X
     //directory existence only checked *once* to avoid race conditions!
     if (resInfo.resolvedPairs.size() != fpCfgList.size())
         throw std::logic_error("Contract violation! " + std::string(__FILE__) + ':' + numberTo<std::string>(__LINE__));
@@ -1061,29 +1087,35 @@ FolderComparison fff::compare(WarningDialogs& warnings,
                 haveFullPair = true;
 
         if (havePartialPair == haveFullPair) //error if: all empty or exist both full and partial pairs -> support single-folder comparison scenario
-            callback.reportWarning(_("A folder input field is empty.") + L" \n\n" + //throw X
-                                   _("The corresponding folder will be considered as empty."), warnings.warnInputFieldEmpty);
+            callback.reportWarning(_("A folder input field is empty.") + L" \n\n" +
+                                   _("The corresponding folder will be considered as empty."), warnings.warnInputFieldEmpty); //throw X
     }
 
-    //check whether one side is a sub directory of the other side (folder-pair-wise!)
-    //similar check (warnDependentBaseFolders) if one directory is read/written by multiple pairs not before beginning of synchronization
+    //Check whether one side is a sub directory of the other side (folder-pair-wise!)
+    //The similar check (warnDependentBaseFolders) if one directory is read/written by multiple pairs not before beginning of synchronization
     {
         std::wstring msg;
+        bool shouldExclude = false;
 
         for (const auto& [folderPair, fpCfg] : workLoad)
             if (std::optional<PathDependency> pd = getPathDependency(folderPair.folderPathLeft,  fpCfg.filter.nameFilter.ref(),
                                                                      folderPair.folderPathRight, fpCfg.filter.nameFilter.ref()))
             {
                 msg += L"\n\n" +
-                       AFS::getDisplayPath(folderPair.folderPathLeft) + L'\n' +
+                       AFS::getDisplayPath(folderPair.folderPathLeft) + L" <-> " + L'\n' +
                        AFS::getDisplayPath(folderPair.folderPathRight);
                 if (!pd->relPath.empty())
-                    msg += L'\n' + _("Exclude:") + L' ' + utfTo<std::wstring>(FILE_NAME_SEPARATOR + pd->relPath + FILE_NAME_SEPARATOR);
+                {
+                    shouldExclude = true;
+                    msg += std::wstring() + L'\n' + L"⇒ " +
+                           _("Exclude:") + L' ' + utfTo<std::wstring>(FILE_NAME_SEPARATOR + pd->relPath + FILE_NAME_SEPARATOR);
+                }
             }
 
         if (!msg.empty())
-            callback.reportWarning(_("One base folder of a folder pair is contained in the other one.") + L'\n' + //throw X
-                                   _("The folder should be excluded from synchronization via filter.") + msg, warnings.warnDependentFolderPair);
+            callback.reportWarning(_("One folder of the folder pair is a subfolder of the other.") +
+                                   (shouldExclude ? L'\n' + _("The folder should be excluded via filter.") : L"") +
+                                   msg, warnings.warnDependentFolderPair); //throw X
     }
     //-------------------end of basic checks------------------------------------------
 
@@ -1092,7 +1124,6 @@ FolderComparison fff::compare(WarningDialogs& warnings,
     {
         std::set<Zstring> folderPathsToLock;
         for (const AbstractPath& folderPath : resInfo.baseFolderStatus.existing)
-
             if (const Zstring& nativePath = getNativeItemPath(folderPath); //restrict directory locking to native paths until further
                 !nativePath.empty())
                 folderPathsToLock.insert(nativePath);
@@ -1125,7 +1156,8 @@ FolderComparison fff::compare(WarningDialogs& warnings,
                 if (fpCfg.compareVar == CompareVariant::content)
                     workLoadByContent.push_back({folderPair, fpCfg});
 
-            std::list<std::shared_ptr<BaseFolderPair>> outputByContent = cmpBuff.compareByContent(workLoadByContent);
+            std::vector<std::shared_ptr<BaseFolderPair>> outputByContent = cmpBuff.compareByContent(workLoadByContent);
+            auto itOByC = outputByContent.begin();
 
             //write output in expected order
             for (const auto& [folderPair, fpCfg] : workLoad)
@@ -1138,12 +1170,9 @@ FolderComparison fff::compare(WarningDialogs& warnings,
                         output.push_back(cmpBuff.compareBySize(folderPair, fpCfg));
                         break;
                     case CompareVariant::content:
-                        assert(!outputByContent.empty());
-                        if (!outputByContent.empty())
-                        {
-                            output.push_back(outputByContent.front());
-                            /**/             outputByContent.pop_front();
-                        }
+                        assert(itOByC != outputByContent.end());
+                        if (itOByC != outputByContent.end())
+                            output.push_back(*itOByC++);
                         break;
                 }
         }
