@@ -3,9 +3,10 @@
 // * GNU General Public License: https://www.gnu.org/licenses/gpl-3.0          *
 // * Copyright (C) Zenju (zenju AT freefilesync DOT org) - All Rights Reserved *
 // *****************************************************************************
+
 #include "dir_lock.h"
-#include <map>
 #include <memory>
+#include <unordered_map>
 #include <zen/crc.h>
 #include <zen/sys_error.h>
 #include <zen/thread.h>
@@ -26,9 +27,9 @@ using namespace fff;
 
 namespace
 {
-const std::chrono::seconds EMIT_LIFE_SIGN_INTERVAL   (5); //show life sign;
-const std::chrono::seconds POLL_LIFE_SIGN_INTERVAL   (4); //poll for life sign;
-const std::chrono::seconds DETECT_ABANDONED_INTERVAL(30); //assume abandoned lock;
+constexpr std::chrono::seconds EMIT_LIFE_SIGN_INTERVAL   (5); //show life sign;
+constexpr std::chrono::seconds POLL_LIFE_SIGN_INTERVAL   (4); //poll for life sign;
+constexpr std::chrono::seconds DETECT_ABANDONED_INTERVAL(30); //assume abandoned lock;
 
 const char LOCK_FILE_DESCR[] = "FreeFileSync";
 const int LOCK_FILE_VERSION = 3; //2020-02-07
@@ -125,7 +126,9 @@ private:
         catch (const SysError& e)
         {
             const std::wstring logMsg = replaceCpy(_("Cannot write file %x."), L"%x", fmtPath(lockFilePath_)) + L' ' + e.toString();
-            std::cerr << utfTo<std::string>(logMsg) << '\n';
+            std::cerr << utfTo<std::string>(logMsg) + '\n';
+
+            warn_static("log on failure!")
         }
     }
 
@@ -166,21 +169,23 @@ struct LockInformation //throw FileError
 
 LockInformation getLockInfoFromCurrentProcess() //throw FileError
 {
-    LockInformation lockInfo = {};
-    lockInfo.lockId = generateGUID();
-    lockInfo.userId = utfTo<std::string>(getLoginUser()); //throw FileError
+    LockInformation lockInfo =
+    {
+        .lockId = generateGUID(),
+        .userId = utfTo<std::string>(getLoginUser()), //throw FileError
+    };
 
     const std::string osName = "Linux";
 
     //wxGetFullHostName() is a performance killer and can hang for some users, so don't touch!
-    std::vector<char> buffer(10000);
-    if (::gethostname(&buffer[0], buffer.size()) != 0)
+    std::vector<char> buf(10000);
+    if (::gethostname(buf.data(), buf.size()) != 0)
         THROW_LAST_FILE_ERROR(_("Cannot get process information."), "gethostname");
-    lockInfo.computerName = osName + ' ' + &buffer[0] + '.';
+    lockInfo.computerName = osName + ' ' + buf.data() + '.';
 
-    if (::getdomainname(&buffer[0], buffer.size()) != 0)
+    if (::getdomainname(buf.data(), buf.size()) != 0)
         THROW_LAST_FILE_ERROR(_("Cannot get process information."), "getdomainname");
-    lockInfo.computerName += &buffer[0];
+    lockInfo.computerName += buf.data(); //can be "(none)"!
 
     lockInfo.processId = ::getpid(); //never fails
 
@@ -195,7 +200,7 @@ LockInformation getLockInfoFromCurrentProcess() //throw FileError
 
 std::string serialize(const LockInformation& lockInfo)
 {
-    MemoryStreamOut<std::string> streamOut;
+    MemoryStreamOut streamOut;
     writeArray(streamOut, LOCK_FILE_DESCR, sizeof(LOCK_FILE_DESCR));
     writeNumber<int32_t>(streamOut, LOCK_FILE_VERSION);
 
@@ -236,7 +241,7 @@ LockInformation unserialize(const std::string& byteStream) //throw SysError
 
     const std::string_view byteStreamTrm = makeStringView(byteStream.begin(), posEnd);
 
-    MemoryStreamOut<std::string> crcStreamOut;
+    MemoryStreamOut crcStreamOut;
     writeNumber<uint32_t>(crcStreamOut, getCrc32(byteStreamTrm.begin(), byteStreamTrm.end() - sizeof(uint32_t)));
 
     if (!endsWith(byteStreamTrm, crcStreamOut.ref()))
@@ -326,7 +331,7 @@ void waitOnDirLock(const Zstring& lockFilePath, const DirLockCallback& notifySta
     {
         const LockInformation& lockInfo = retrieveLockInfo(lockFilePath); //throw FileError
 
-        infoMsg += L" | " + _("User name:") +  L' ' + utfTo<std::wstring>(lockInfo.userId);
+        infoMsg += SPACED_DASH + _("Username:") +  L' ' + utfTo<std::wstring>(lockInfo.userId);
 
         originalLockId = lockInfo.lockId;
         switch (getProcessStatus(lockInfo)) //throw FileError
@@ -397,7 +402,7 @@ void waitOnDirLock(const Zstring& lockFilePath, const DirLockCallback& notifySta
                 if (lastCheckTime >= lastLifeSign + EMIT_LIFE_SIGN_INTERVAL + std::chrono::seconds(1))
                 {
                     const int remainingSeconds = std::max(0, static_cast<int>(std::chrono::duration_cast<std::chrono::seconds>(DETECT_ABANDONED_INTERVAL - (now - lastLifeSign)).count()));
-                    notifyStatus(infoMsg + L" | " + _("Detecting abandoned lock...") + L' ' + _P("1 sec", "%x sec", remainingSeconds)); //throw X
+                    notifyStatus(infoMsg + SPACED_DASH + _("Detecting abandoned lock...") + L' ' + _P("1 sec", "%x sec", remainingSeconds)); //throw X
                 }
                 else
                     notifyStatus(std::wstring(infoMsg)); //throw X; emit a message in any case (might clear other one)
@@ -415,6 +420,7 @@ void releaseLock(const Zstring& lockFilePath) //noexcept
         removeFilePlain(lockFilePath); //throw FileError
     }
     catch (FileError&) {}
+    warn_static("log!!! at the very least") //https://freefilesync.org/forum/viewtopic.php?t=7655
 }
 
 
@@ -438,13 +444,18 @@ bool tryLock(const Zstring& lockFilePath) //throw FileError
 
         THROW_LAST_FILE_ERROR(replaceCpy(_("Cannot write file %x."), L"%x", fmtPath(lockFilePath)), "open");
     }
-    FileOutput fileOut(hFile, lockFilePath, nullptr /*notifyUnbufferedIO*/); //pass handle ownership
+    FileOutputPlain fileOut(hFile, lockFilePath); //pass handle ownership
 
     //write housekeeping info: user, process info, lock GUID
     const std::string byteStream = serialize(getLockInfoFromCurrentProcess()); //throw FileError
 
-    fileOut.write(byteStream.c_str(), byteStream.size()); //throw FileError, (X)
-    fileOut.finalize();                                   //
+    unbufferedSave(byteStream, [&](const void* buffer, size_t bytesToWrite)
+    {
+        return fileOut.tryWrite(buffer, bytesToWrite); //throw FileError; may return short! CONTRACT: bytesToWrite > 0
+    },
+    fileOut.getBlockSize());
+
+    fileOut.close(); //throw FileError
     return true;
 }
 }
@@ -500,8 +511,8 @@ public:
         tidyUp();
 
         //optimization: check if we already own a lock for this path
-        auto itGuid = guidByPath_.find(lockFilePath);
-        if (itGuid != guidByPath_.end())
+        if (auto itGuid = guidByPath_.find(lockFilePath);
+            itGuid != guidByPath_.end())
             if (const std::shared_ptr<SharedDirLock>& activeLock = getActiveLock(itGuid->second)) //returns null-lock if not found
                 return activeLock; //SharedDirLock is still active -> enlarge circle of shared ownership
 
@@ -545,8 +556,8 @@ private:
         std::erase_if(guidByPath_, [&](const auto& v) { return !locksByGuid_.contains(v.second); });
     }
 
-    std::map<Zstring, UniqueId> guidByPath_;                      //lockFilePath |-> GUID; n:1; locks can be referenced by a lockFilePath or alternatively a GUID
-    std::map<UniqueId, std::weak_ptr<SharedDirLock>> locksByGuid_; //GUID |-> "shared lock ownership"; 1:1
+    std::unordered_map<Zstring, UniqueId> guidByPath_;                      //lockFilePath |-> GUID; n:1; locks can be referenced by a lockFilePath or alternatively a GUID
+    std::unordered_map<UniqueId, std::weak_ptr<SharedDirLock>> locksByGuid_; //GUID |-> "shared lock ownership"; 1:1
 };
 
 
