@@ -11,7 +11,6 @@
 
     #include <iostream> //std::cerr
 
-
 using namespace zen;
 
 
@@ -31,7 +30,7 @@ Zstring getProcessParentFolderPath()
         }
         catch (const FileError& e)
         {
-            throw std::runtime_error(std::string(__FILE__) + '[' + numberTo<std::string>(__LINE__) + "] " + utfTo<std::string>(e.toString()));
+            throw std::runtime_error(std::string(__FILE__) + '[' + numberTo<std::string>(__LINE__) + "] Failed to get process parent folder. " + utfTo<std::string>(e.toString()));
         }
     }();
     return exeFolderParentPath;
@@ -48,51 +47,51 @@ Zstring fff::getInstallDirPath()
 
 
 
-Zstring fff::getResourceDirPf()
+Zstring fff::getResourceDirPath()
 {
-    return getProcessParentFolderPath() + FILE_NAME_SEPARATOR + Zstr("Resources") + FILE_NAME_SEPARATOR;
+    return appendPath(getProcessParentFolderPath(), Zstr("Resources"));
 }
 
 
-Zstring fff::getConfigDirPathPf()
+Zstring fff::getConfigDirPath()
 {
     //note: compiler generates magic-statics code => fine, we don't expect accesses during shutdown
-    static const Zstring ffsConfigPathPf = []
+    static const Zstring ffsConfigPath = []
     {
         /*  Windows:            %AppData%\FreeFileSync
             macOS:              ~/Library/Application Support/FreeFileSync
             Linux (XDG layout): ~/.config/FreeFileSync                        */
-        const Zstring& ffsConfigPath = []
+        const Zstring& configPath = []
         {
             try
             {
-                return
-                appendSeparator(getUserDataPath()) + Zstr("FreeFileSync"); //throw FileError
+                return appendPath(getUserDataPath(), Zstr("FreeFileSync")); //throw FileError
             }
             catch (const FileError& e)
             {
-                throw std::runtime_error(std::string(__FILE__) + '[' + numberTo<std::string>(__LINE__) + "] " + utfTo<std::string>(e.toString()));
+                throw std::runtime_error(std::string(__FILE__) + '[' + numberTo<std::string>(__LINE__) + "] Failed to get config path. " + utfTo<std::string>(e.toString()));
             }
         }();
 
-        try //create the config folder if not existing + create "Logs" subfolder while we're at it
+        try
         {
-            createDirectoryIfMissingRecursion(appendSeparator(ffsConfigPath) + Zstr("Logs")); //throw FileError
+            createDirectoryIfMissingRecursion(configPath); //throw FileError
         }
         catch (const FileError& e)
         {
             assert(false);
-            std::cerr << utfTo<std::string>(e.toString()) << '\n';
+            std::cerr << utfTo<std::string>(e.toString()) + '\n';
+            warn_static("at least log on failure!")
         }
-        return appendSeparator(ffsConfigPath);
+        return configPath;
     }();
-    return ffsConfigPathPf;
+    return ffsConfigPath;
 }
 
 
 //this function is called by RealTimeSync!!!
 Zstring fff::getFreeFileSyncLauncherPath()
 {
-    return getProcessParentFolderPath() + Zstr("/FreeFileSync");
+    return appendPath(getProcessParentFolderPath(), Zstr("FreeFileSync"));
 
 }
