@@ -9,7 +9,6 @@
 #include <zen/http.h>
 #include <zen/sys_info.h>
 #include <wx/datetime.h>
-#include "ffs_paths.h"
 #include "afs/concrete.h"
 
 using namespace zen;
@@ -19,13 +18,20 @@ using AFS = AbstractFileSystem;
 
 namespace
 {
-const int LOG_PREVIEW_FAIL_MAX = 25;
+const int LOG_PREVIEW_MAX = 25;
+
+const int EMAIL_PREVIEW_MAX = LOG_PREVIEW_MAX;
+const int EMAIL_ITEMS_MAX = 250;
+
+const int EMAIL_SHORT_PREVIEW_MAX = 5; //summary email
+const int EMAIL_SHORT_ITEMS_MAX   = 0; //
+
 const int SEPARATION_LINE_LEN = 40;
 
 
-std::string generateLogHeaderTxt(const ProcessSummary& s, const ErrorLog& log, int logPreviewFailsMax)
+std::string generateLogHeaderTxt(const ProcessSummary& s, const ErrorLog& log, int logPreviewMax)
 {
-    const std::string tabSpace(4, ' '); //4: the only sensible space count for tabs
+    const auto tabSpace = utfTo<std::string>(TAB_SPACE);
 
     std::string headerLine;
     for (const std::wstring& jobName : s.jobNames)
@@ -34,7 +40,7 @@ std::string generateLogHeaderTxt(const ProcessSummary& s, const ErrorLog& log, i
     if (!headerLine.empty())
         headerLine += ' ';
 
-    const TimeComp tc = getLocalTime(std::chrono::system_clock::to_time_t(s.startTime)); //returns empty string on failure
+    const TimeComp tc = getLocalTime(std::chrono::system_clock::to_time_t(s.startTime)); //returns TimeComp() on error
     headerLine += utfTo<std::string>(formatTime(formatDateTag, tc) + Zstr(" [") + formatTime(formatTimeTag, tc) + Zstr(']'));
 
     //assemble summary box
@@ -43,7 +49,7 @@ std::string generateLogHeaderTxt(const ProcessSummary& s, const ErrorLog& log, i
     summary.push_back(tabSpace + utfTo<std::string>(getSyncResultLabel(s.syncResult)));
     summary.emplace_back();
 
-    const ErrorLog::Stats logCount = log.getStats();
+    const ErrorLogStats logCount = getStats(log);
 
     if (logCount.error   > 0) summary.push_back(tabSpace + utfTo<std::string>(_("Errors:")   + L' ' + formatNumber(logCount.error)));
     if (logCount.warning > 0) summary.push_back(tabSpace + utfTo<std::string>(_("Warnings:") + L' ' + formatNumber(logCount.warning)));
@@ -81,14 +87,14 @@ std::string generateLogHeaderTxt(const ProcessSummary& s, const ErrorLog& log, i
         output += std::string(SEPARATION_LINE_LEN, '_') + '\n';
 
         int previewCount = 0;
-        if (logPreviewFailsMax > 0)
-            for (const LogEntry& entry : log)
-                if (entry.type & (MSG_TYPE_WARNING | MSG_TYPE_ERROR))
-                {
-                    output += utfTo<std::string>(formatMessage(entry));
-                    if (++previewCount >= logPreviewFailsMax)
-                        break;
-                }
+        for (const LogEntry& entry : log)
+            if (entry.type & (MSG_TYPE_WARNING | MSG_TYPE_ERROR))
+            {
+                if (previewCount++ >= logPreviewMax)
+                    break;
+                output += utfTo<std::string>(formatMessage(entry));
+            }
+
         if (logFailTotal > previewCount)
             output += "  [...]  " + utfTo<std::string>(replaceCpy(_P("Showing %y of 1 item", "Showing %y of %x items", logFailTotal), //%x used as plural form placeholder!
                                                                   L"%y", formatNumber(previewCount))) + '\n';
@@ -98,23 +104,25 @@ std::string generateLogHeaderTxt(const ProcessSummary& s, const ErrorLog& log, i
 }
 
 
-std::string generateLogFooterTxt(const std::wstring& logFilePath, int logItemsTotal, int logPreviewItemsMax) //throw FileError
+std::string generateLogFooterTxt(const std::wstring& logFilePath /*optional*/, int logItemsTotal, int logItemsMax) //throw FileError
 {
     const ComputerModel cm = getComputerModel(); //throw FileError
 
     std::string output;
-    if (logItemsTotal > logPreviewItemsMax)
+    if (logItemsTotal > logItemsMax)
         output += "  [...]  " + utfTo<std::string>(replaceCpy(_P("Showing %y of 1 item", "Showing %y of %x items", logItemsTotal), //%x used as plural form placeholder!
-                                                              L"%y", formatNumber(logPreviewItemsMax))) + '\n';
+                                                              L"%y", formatNumber(logItemsMax))) + '\n';
 
-    return output += '\n' + std::string(SEPARATION_LINE_LEN, '_') + '\n' +
+    output += std::string(SEPARATION_LINE_LEN, '_') + '\n' +
 
-                     utfTo<std::string>(getOsDescription() + /*throw FileError*/ +
-                                        L" [" + utfTo<std::wstring>(getLoginUser()) /*throw FileError*/ + L']' +
-                                        (!cm.model .empty() ? L" - " + cm.model  : L"") +
-                                        (!cm.vendor.empty() ? L" - " + cm.vendor : L"") + L'\n' +
+              utfTo<std::string>(getOsDescription() + /*throw FileError*/ +
+                                 L" - " + utfTo<std::wstring>(getUserDescription()) /*throw FileError*/ +
+                                 (!cm.model .empty() ? L" - " + cm.model  : L"") +
+                                 (!cm.vendor.empty() ? L" - " + cm.vendor : L"")) + '\n';
+    if (!logFilePath.empty())
+        output += utfTo<std::string>(_("Log file:") + L' ' + logFilePath) + '\n';
 
-                                        _("Log file:") + L' ' + logFilePath) + '\n';
+    return output;
 }
 
 
@@ -195,7 +203,7 @@ std::wstring generateLogTitle(const ProcessSummary& s)
 }
 
 
-std::string generateLogHeaderHtml(const ProcessSummary& s, const ErrorLog& log, int logPreviewFailsMax)
+std::string generateLogHeaderHtml(const ProcessSummary& s, const ErrorLog& log, int logPreviewMax)
 {
     std::string output = R"(<!DOCTYPE html>
 <html lang="en">
@@ -222,7 +230,7 @@ std::string generateLogHeaderHtml(const ProcessSummary& s, const ErrorLog& log,
     for (const std::wstring& jobName : s.jobNames)
         jobNamesFmt += (jobNamesFmt.empty() ? "" : " + ") + htmlTxt(jobName);
 
-    const TimeComp tc = getLocalTime(std::chrono::system_clock::to_time_t(s.startTime)); //returns empty string on failure
+    const TimeComp tc = getLocalTime(std::chrono::system_clock::to_time_t(s.startTime)); //returns TimeComp() on error
     output += R"(	<div><span style="font-weight:600; color:gray;">)" + jobNamesFmt + R"(</span> &nbsp;<span style="white-space:nowrap">)" +
               htmlTxt(formatTime(formatDateTag, tc)) + " &nbsp;" + htmlTxt(formatTime(formatTimeTag, tc)) + "</span></div>\n";
 
@@ -242,7 +250,7 @@ std::string generateLogHeaderHtml(const ProcessSummary& s, const ErrorLog& log,
         </div>
         <table role="presentation" class="summary-table" style="border-spacing:0; margin-left:10px; padding:5px 10px;">)";
 
-    const ErrorLog::Stats logCount = log.getStats();
+    const ErrorLogStats logCount = getStats(log);
 
     if (logCount.error > 0) 
         output += R"(
@@ -301,14 +309,14 @@ std::string generateLogHeaderHtml(const ProcessSummary& s, const ErrorLog& log,
     <table class="log-items" style="line-height:1em; border-spacing:0;">
 )";
         int previewCount = 0;
-        if (logPreviewFailsMax > 0)
-            for (const LogEntry& entry : log)
-                if (entry.type & (MSG_TYPE_WARNING | MSG_TYPE_ERROR))
-                {
-                    output += formatMessageHtml(entry);
-                    if (++previewCount >= logPreviewFailsMax)
-                        break;
-                }
+        for (const LogEntry& entry : log)
+            if (entry.type & (MSG_TYPE_WARNING | MSG_TYPE_ERROR))
+            {
+                if (previewCount++ >= logPreviewMax)
+                    break;
+                output += formatMessageHtml(entry);
+            }
+
         output += R"(	</table>
 )";
         if (logFailTotal > previewCount)
@@ -327,7 +335,7 @@ std::string generateLogHeaderHtml(const ProcessSummary& s, const ErrorLog& log,
 }
 
 
-std::string generateLogFooterHtml(const std::wstring& logFilePath, int logItemsTotal, int logPreviewItemsMax) //throw FileError
+std::string generateLogFooterHtml(const std::wstring& logFilePath /*optional*/, int logItemsTotal, int logItemsMax) //throw FileError
 {
     const std::string osImage = "os-linux.png";
     const ComputerModel cm = getComputerModel(); //throw FileError
@@ -335,63 +343,71 @@ std::string generateLogFooterHtml(const std::wstring& logFilePath, int logItemsT
     std::string output = R"(	</table>
 )";
 
-    if (logItemsTotal > logPreviewItemsMax)
+    if (logItemsTotal > logItemsMax)
         output += R"(	<div><span style="font-weight:600; padding:0 10px;">[&hellip;]</span>)" + 
                   htmlTxt(replaceCpy(_P("Showing %y of 1 item", "Showing %y of %x items", logItemsTotal), //%x used as plural form placeholder!
-                          L"%y", formatNumber(logPreviewItemsMax))) + "</div>\n";
-
-    return output += R"(	<br>
+                          L"%y", formatNumber(logItemsMax))) + "</div>\n";
 
+    output += R"(
     <div style="border-bottom:1px solid #AAA; margin:5px 0;"></div>
     <div style="font-size:small;">
         <img src="https://freefilesync.org/images/log/)" + osImage + R"(" width="24" height="24" alt="" style="vertical-align:middle;">
         <span style="vertical-align:middle;">)" + htmlTxt(getOsDescription()) + /*throw FileError*/ + 
-            " [" + htmlTxt(getLoginUser()) /*throw FileError*/ + ']' +
+            " &ndash; " + htmlTxt(getUserDescription()) /*throw FileError*/ + 
             (!cm.model .empty() ? " &ndash; " + htmlTxt(cm.model ) : "") +
             (!cm.vendor.empty() ? " &ndash; " + htmlTxt(cm.vendor) : "") + R"(</span>
-    </div>
+    </div>)";
+
+    if (!logFilePath.empty())
+        output += R"(
     <div style="font-size:small;">
         <img src="https://freefilesync.org/images/log/log.png" width="24" height="24" alt=")" + htmlTxt(_("Log file:")) + R"(" style="vertical-align:middle;">
         <span style="font-family: Consolas,'Courier New',Courier,monospace; vertical-align:middle;">)" + htmlTxt(logFilePath) + R"(</span>
-    </div>
+    </div>)";
+
+    output += R"(
 </body>
 </html>
 )";
+    return output;
 }
 
 //-> Astyle fucks up! => no INDENT-ON
 
 
-void streamToLogFile(const ProcessSummary& summary, //throw FileError
-                     const ErrorLog& log,
-                     AFS::OutputStream& streamOut,
-                     const AbstractPath& logFilePath,
-                     LogFileFormat logFormat)
+//write log items in blocks instead of creating one big string: memory allocation might fail; think 1 million entries!
+template <class Function> 
+void streamToLogFile(const ProcessSummary& summary, const ErrorLog& log,
+                     int logPreviewMax, int logItemsMax, 
+                     const std::wstring& logFilePath /*optional*/,
+                     LogFileFormat logFormat, Function stringOut /*(const std::string& s); throw X*/) //throw SysError, X
 {
-    const int logItemsTotal = log.end() - log.begin();
-    const int logPreviewItemsMax = std::numeric_limits<int>::max();
+    stringOut(logFormat == LogFileFormat::html ?
+              generateLogHeaderHtml(summary, log, logPreviewMax) :
+              generateLogHeaderTxt (summary, log, logPreviewMax)); //throw X
 
-    std::string buffer = logFormat == LogFileFormat::html ? 
-                         generateLogHeaderHtml(summary, log, LOG_PREVIEW_FAIL_MAX) :
-                         generateLogHeaderTxt (summary, log, LOG_PREVIEW_FAIL_MAX);
-
-    //write log items in blocks instead of creating one big string: memory allocation might fail; think 1 million entries!
+    int itemCount = 0;
     for (const LogEntry& entry : log)
     {
-        buffer += logFormat == LogFileFormat::html ?
-                  formatMessageHtml(entry) :
-                  formatMessage    (entry);
-
-        streamOut.write(&buffer[0], buffer.size()); //throw FileError, X
-        buffer.clear();
+        if (itemCount++ >= logItemsMax)
+            break;
+        stringOut(logFormat == LogFileFormat::html ?
+                    formatMessageHtml(entry) :
+                    formatMessage    (entry)); //throw X
     }
 
-    buffer += logFormat == LogFileFormat::html ? 
-              generateLogFooterHtml(AFS::getDisplayPath(logFilePath), logItemsTotal, logPreviewItemsMax) : //throw FileError
-              generateLogFooterTxt (AFS::getDisplayPath(logFilePath), logItemsTotal, logPreviewItemsMax);  //throw FileError
+    const std::string footer = [&]
+    {
+        try
+        {
+            return logFormat == LogFileFormat::html ?
+                   generateLogFooterHtml(logFilePath, static_cast<int>(log.size()), logItemsMax): //throw FileError
+                   generateLogFooterTxt (logFilePath, static_cast<int>(log.size()), logItemsMax); //
+        }
+        catch (const FileError& e) { throw SysError(replaceCpy(e.toString(), L"\n\n", L'\n')); } //errors should be further enriched by context info => SysError
+    }(); //caveat: don't catch exceptions thrown by stringOut()!
 
-    //don't forget to flush:
-    streamOut.write(&buffer[0], buffer.size()); //throw FileError, X
+    stringOut(footer); //throw X
 }
 
 
@@ -417,15 +433,36 @@ void saveNewLogFile(const AbstractPath& logFilePath, //throw FileError, X
                                bytesWritten_ = int64_t(0),
                                msg_ = replaceCpy(_("Saving file %x..."), L"%x", fmtPath(AFS::getDisplayPath(logFilePath)))]
          (int64_t bytesDelta) mutable
-    {
-        if (notifyStatus)
-            notifyStatus(msg_ + L" (" + formatFilesizeShort(bytesWritten_ += bytesDelta) + L')'); //throw X
-    };
+         {
+              if (notifyStatus)
+                  notifyStatus(msg_ + L" (" + formatFilesizeShort(bytesWritten_ += bytesDelta) + L')'); //throw X
+         };
 
     //already existing: undefined behavior! (e.g. fail/overwrite/auto-rename)
-    std::unique_ptr<AFS::OutputStream> logFileStream = AFS::getOutputStream(logFilePath, std::nullopt /*streamSize*/, std::nullopt /*modTime*/, notifyUnbufferedIO); //throw FileError
-    streamToLogFile(summary, log, *logFileStream, logFilePath, logFormat); //throw FileError, X
-    logFileStream->finalize();                     //throw FileError, X
+    std::unique_ptr<AFS::OutputStream> logFileOut = AFS::getOutputStream(logFilePath, 
+                                                                         std::nullopt /*streamSize*/, 
+                                                                         std::nullopt /*modTime*/); //throw FileError
+
+    BufferedOutputStream streamOut([&](const void* buffer, size_t bytesToWrite)
+    {
+        return logFileOut->tryWrite(buffer, bytesToWrite, notifyUnbufferedIO); //throw FileError, X
+    },
+    logFileOut->getBlockSize());
+
+    try
+    {
+        streamToLogFile(summary, log, LOG_PREVIEW_MAX, std::numeric_limits<int>::max() /*logItemsMax*/, 
+                        std::wstring() /*logFilePath -> superfluous*/, logFormat, 
+                        [&](const std::string& str){ streamOut.write(str.data(), str.size()); } /*throw FileError, X*/); //throw SysError, FileError, X
+    }
+    catch (const SysError& e)
+    {
+        throw FileError(replaceCpy(_("Cannot write file %x."), L"%x", fmtPath(AFS::getDisplayPath(logFilePath))), e.toString());
+    }
+
+    streamOut.flushBuffer(); //throw FileError, X
+
+    logFileOut->finalize(notifyUnbufferedIO); //throw FileError, X
 }
 
 
@@ -444,7 +481,7 @@ std::vector<LogFileInfo> getLogFiles(const AbstractPath& logFolderPath) //throw
 {
     std::vector<LogFileInfo> logfiles;
 
-    AFS::traverseFolderFlat(logFolderPath, [&](const AFS::FileInfo& fi) //throw FileError
+    AFS::traverseFolder(logFolderPath, [&](const AFS::FileInfo& fi) //throw FileError
     {
         //"Backup FreeFileSync 2013-09-15 015052.123.html"
         //"Jobname1 + Jobname2 2013-09-15 015052.123.log"
@@ -454,32 +491,29 @@ std::vector<LogFileInfo> getLogFiles(const AbstractPath& logFolderPath) //throw
         if (endsWith(fi.itemName, Zstr(".log")) || //case-sensitive: e.g. ".LOG" is not from FFS, right?
             endsWith(fi.itemName, Zstr(".html")))
         {
-            auto tsBegin = fi.itemName.begin();
-            auto tsEnd   = tsBegin + fi.itemName.rfind('.');
+            ZstringView itemPhrase = beforeLast<ZstringView>(fi.itemName, Zstr('.'), IfNotFoundReturn::none);
+            
+            if (endsWith(itemPhrase, STATUS_END_TOKEN))
+                itemPhrase = beforeLast(itemPhrase, STATUS_BEGIN_TOKEN, IfNotFoundReturn::all);
 
-            if (tsBegin != tsEnd && tsEnd[-1] == STATUS_END_TOKEN)
-                tsEnd = searchLast(tsBegin, tsEnd,
-                                   std::begin(STATUS_BEGIN_TOKEN), std::end(STATUS_BEGIN_TOKEN) - 1);
-
-            if (tsEnd - tsBegin >= TIME_STAMP_LENGTH &&
-                tsEnd[-4] == Zstr('.') &&
-                isdigit(tsEnd[-3]) &&
-                isdigit(tsEnd[-2]) &&
-                isdigit(tsEnd[-1]))
+            if (itemPhrase.size() >= TIME_STAMP_LENGTH &&
+                itemPhrase.end()[-4] == Zstr('.') &&
+                isdigit(itemPhrase.end()[-3]) &&
+                isdigit(itemPhrase.end()[-2]) &&
+                isdigit(itemPhrase.end()[-1]))
             {
-                tsBegin = tsEnd - TIME_STAMP_LENGTH;
-                const TimeComp tc = parseTime(Zstr("%Y-%m-%d %H%M%S"), makeStringView(tsBegin, 17)); //returns TimeComp() on error
-                const time_t t = localToTimeT(tc); //returns -1 on error
-                if (t != -1)
+                const TimeComp tc = parseTime(Zstr("%Y-%m-%d %H%M%S"), makeStringView(itemPhrase.end() - TIME_STAMP_LENGTH, 17)); //returns TimeComp() on error
+                if (const auto [localTime, timeValid] = localToTimeT(tc);
+                    timeValid)
                 {
-                    Zstring jobNames(fi.itemName.begin(), tsBegin);
-                    if (!jobNames.empty())
+                    itemPhrase.remove_suffix(TIME_STAMP_LENGTH);
+                    if (!itemPhrase.empty())
                     {
-                        assert(jobNames.size() >= 2 && endsWith(jobNames, Zstr(' ')));
-                        jobNames.pop_back();
+                        assert(itemPhrase.size() >= 2 && endsWith(itemPhrase, Zstr(' ')));
+                        itemPhrase = trimCpy(itemPhrase);
                     }
 
-                    logfiles.push_back({AFS::appendRelPath(logFolderPath, fi.itemName), t, utfTo<std::wstring>(jobNames)});
+                    logfiles.push_back({AFS::appendRelPath(logFolderPath, fi.itemName), localTime, utfTo<std::wstring>(itemPhrase)});
                 }
             }
         }
@@ -510,7 +544,7 @@ void limitLogfileCount(const AbstractPath& logFolderPath, //throw FileError, X
             tc.second = 0;
             tc.minute = 0;
             tc.hour   = 0;
-            return localToTimeT(tc); //returns -1 on error => swallow => no versions trimmed by versionMaxAgeDays
+            return localToTimeT(tc).first; //0 on error => swallow => no versions trimmed by versionMaxAgeDays
         }();
         const time_t cutOffTime = lastMidnightTime - static_cast<time_t>(logfilesMaxAgeDays) * 24 * 3600;
 
@@ -536,15 +570,12 @@ void limitLogfileCount(const AbstractPath& logFolderPath, //throw FileError, X
 }
 
 
-Zstring fff::getLogFolderDefaultPath() { return getConfigDirPathPf() + Zstr("Logs") ; }
-
-
 //"Backup FreeFileSync 2013-09-15 015052.123.html"
 //"Backup FreeFileSync 2013-09-15 015052.123 [Error].html"
 //"Backup FreeFileSync + RealTimeSync 2013-09-15 015052.123 [Error].log"
-AbstractPath fff::generateLogFilePath(LogFileFormat logFormat, const ProcessSummary& summary, const Zstring& altLogFolderPathPhrase /*optional*/)
+Zstring fff::generateLogFileName(LogFileFormat logFormat, const ProcessSummary& summary)
 {
-    //const std::string colon = "\xcb\xb8"; //="modifier letter raised colon" => regular colon is forbidden in file names on Windows and OS X
+    //const std::string colon = "\xcb\xb8"; //="modifier letter raised colon" => regular colon is forbidden in file names on Windows and macOS
     //=> too many issues, most notably cmd.exe is not Unicode-aware: https://freefilesync.org/forum/viewtopic.php?t=1679
 
     Zstring jobNamesFmt;
@@ -565,7 +596,7 @@ AbstractPath fff::generateLogFilePath(LogFileFormat logFormat, const ProcessSumm
 
     const TimeComp tc = getLocalTime(std::chrono::system_clock::to_time_t(summary.startTime));
     if (tc == TimeComp())
-        throw FileError(L"Failed to determine current time: " + numberTo<std::wstring>(summary.startTime.time_since_epoch().count()));
+        throw FileError(L"Failed to determine current time: (time_t) " + numberTo<std::wstring>(summary.startTime.time_since_epoch().count()));
 
     const auto timeMs = std::chrono::duration_cast<std::chrono::milliseconds>(summary.startTime.time_since_epoch()).count() % 1000;
     assert(std::chrono::duration_cast<std::chrono::seconds>(summary.startTime.time_since_epoch()).count() == std::chrono::system_clock::to_time_t(summary.startTime));
@@ -594,16 +625,9 @@ AbstractPath fff::generateLogFilePath(LogFileFormat logFormat, const ProcessSumm
     if (!failStatus.empty())
         logFileName += STATUS_BEGIN_TOKEN + utfTo<Zstring>(failStatus) + STATUS_END_TOKEN;
 
-    if (logFormat == LogFileFormat::html)
-        logFileName += Zstr(".html");
-    else
-        logFileName += Zstr(".log");
-
-    AbstractPath logFolderPath = createAbstractPath(altLogFolderPathPhrase);
-    if (AFS::isNullPath(logFolderPath))
-        logFolderPath = createAbstractPath(getLogFolderDefaultPath());
-
-    return AFS::appendRelPath(logFolderPath, logFileName);
+    logFileName += logFormat == LogFileFormat::html ? Zstr(".html") : Zstr(".log");
+    
+    return logFileName;
 }
 
 
