@@ -6,7 +6,6 @@
 
 #include "abstract_folder_picker.h"
 #include <wx+/async_task.h>
-#include <wx+/std_button_layout.h>
 #include <wx+/image_resources.h>
 #include <wx+/popup_dlg.h>
 #include <wx+/image_tools.h>
@@ -90,14 +89,14 @@ AbstractFolderPickerDlg::AbstractFolderPickerDlg(wxWindow* parent, AbstractPath&
     m_staticTextStatus->SetLabel(L"");
     m_treeCtrlFileSystem->SetMinSize({fastFromDIP(350), fastFromDIP(400)});
 
-    const int iconSize = IconBuffer::getSize(IconBuffer::SIZE_SMALL);
+    const int iconSize = IconBuffer::getSize(IconBuffer::IconSize::small);
     auto imgList = std::make_unique<wxImageList>(iconSize, iconSize);
 
     //add images in same sequence like TreeNodeImage enum!!!
     imgList->Add(loadImage("server", iconSize));
-    imgList->Add(        IconBuffer::genericDirIcon(IconBuffer::SIZE_SMALL));
-    imgList->Add(layOver(IconBuffer::genericDirIcon(IconBuffer::SIZE_SMALL),
-                         IconBuffer::linkOverlayIcon(IconBuffer::SIZE_SMALL)));
+    imgList->Add(        IconBuffer::genericDirIcon (IconBuffer::IconSize::small));
+    imgList->Add(layOver(IconBuffer::genericDirIcon (IconBuffer::IconSize::small),
+                         IconBuffer::linkOverlayIcon(IconBuffer::IconSize::small)));
     imgList->Add(loadImage("msg_error", iconSize));
     assert(imgList->GetImageCount() == static_cast<int>(TreeNodeImage::error) + 1);
 
@@ -117,7 +116,7 @@ AbstractFolderPickerDlg::AbstractFolderPickerDlg(wxWindow* parent, AbstractPath&
             //1. test server connection:
             const AFS::ItemType type = AFS::getItemType(folderPath); //throw FileError
             //2. navigate + select path
-            navigateToExistingPath(rootId, split(folderPath.afsPath.value, FILE_NAME_SEPARATOR, SplitOnEmpty::skip), type);
+            navigateToExistingPath(rootId, splitCpy(folderPath.afsPath.value, FILE_NAME_SEPARATOR, SplitOnEmpty::skip), type);
         }
         catch (const FileError& e) //not existing or access error
         {
@@ -148,7 +147,7 @@ struct FlatTraverserCallback : public AFS::TraverserCallback
 {
     struct Result
     {
-        std::map<Zstring, bool /*is symlink*/> folderNames;
+        std::unordered_map<Zstring, bool /*is symlink*/> folderNames;
         std::wstring errorMsg;
     };
 
@@ -253,7 +252,7 @@ void AbstractFolderPickerDlg::findAndNavigateToExistingPath(const AbstractPath&
     if (!AFS::getParentPath(folderPath))
         return m_staticTextStatus->SetLabel(L"");
 
-    m_staticTextStatus->SetLabel(_("Scanning...") + L' ' + utfTo<std::wstring>(FILE_NAME_SEPARATOR + folderPath.afsPath.value)); //keep it short!
+    m_staticTextStatus->SetLabelText(_("Scanning...") + L' ' + utfTo<std::wstring>(FILE_NAME_SEPARATOR + folderPath.afsPath.value)); //keep it short!
 
     guiQueue_.processAsync([folderPath]() -> std::optional<AFS::ItemType>
     {
@@ -269,7 +268,7 @@ void AbstractFolderPickerDlg::findAndNavigateToExistingPath(const AbstractPath&
         if (type)
         {
             m_staticTextStatus->SetLabel(L"");
-            navigateToExistingPath(m_treeCtrlFileSystem->GetRootItem(), split(folderPath.afsPath.value, FILE_NAME_SEPARATOR, SplitOnEmpty::skip), *type);
+            navigateToExistingPath(m_treeCtrlFileSystem->GetRootItem(), splitCpy(folderPath.afsPath.value, FILE_NAME_SEPARATOR, SplitOnEmpty::skip), *type);
         }
         else //split into multiple small async tasks rather than a single large one!
             findAndNavigateToExistingPath(*AFS::getParentPath(folderPath));
