@@ -11,8 +11,8 @@
 #include <wx/app.h>
 #include <wx/sound.h>
 #include "../afs/concrete.h"
-#include "../log_file.h"
-#include "../fatal_error.h"
+#include "../config.h"
+//#include "../log_file.h"
 
 using namespace zen;
 using namespace fff;
@@ -25,6 +25,7 @@ BatchStatusHandler::BatchStatusHandler(bool showProgress,
                                        size_t autoRetryCount,
                                        std::chrono::seconds autoRetryDelay,
                                        const Zstring& soundFileSyncComplete,
+                                       const Zstring& soundFileAlertPending,
                                        wxSize progressDlgSize, bool dlgMaximize,
                                        bool autoCloseDialog,
                                        PostSyncAction postSyncAction,
@@ -34,8 +35,9 @@ BatchStatusHandler::BatchStatusHandler(bool showProgress,
     autoRetryCount_(autoRetryCount),
     autoRetryDelay_(autoRetryDelay),
     soundFileSyncComplete_(soundFileSyncComplete),
+    soundFileAlertPending_(soundFileAlertPending),
     progressDlg_(SyncProgressDialog::create(progressDlgSize, dlgMaximize, [this] { userRequestAbort(); }, *this, nullptr /*parentWindow*/, showProgress, autoCloseDialog,
-{jobName}, startTime, ignoreErrors, autoRetryCount, [&]
+{jobName}, std::chrono::system_clock::to_time_t(startTime), ignoreErrors, autoRetryCount, [&]
 {
     switch (postSyncAction)
     {
@@ -64,30 +66,29 @@ BatchStatusHandler::~BatchStatusHandler()
 
 
 BatchStatusHandler::Result BatchStatusHandler::reportResults(const Zstring& postSyncCommand, PostSyncCondition postSyncCondition,
-                                                             const Zstring& altLogFolderPathPhrase, int logfilesMaxAgeDays, LogFileFormat logFormat,
+                                                             const AbstractPath& logFolderPath, int logfilesMaxAgeDays, LogFileFormat logFormat,
                                                              const std::set<AbstractPath>& logFilePathsToKeep,
                                                              const std::string& emailNotifyAddress, ResultsNotification emailNotifyCondition) //noexcept!!
 {
-    const auto totalTime = std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::system_clock::now() - startTime_);
-
-    progressDlg_->timerSetStatus(false /*active*/); //keep correct summary window stats considering count down timer, system sleep
+    //keep correct summary window stats considering count down timer, system sleep
+    const std::chrono::milliseconds totalTime = progressDlg_->pauseAndGetTotalTime();
 
     //determine post-sync status irrespective of further errors during tear-down
     const SyncResult syncResult = [&]
     {
         if (getAbortStatus())
         {
-            errorLog_.logMsg(_("Stopped"), MSG_TYPE_ERROR); //= user cancel
+            logMsg(errorLog_, _("Stopped"), MSG_TYPE_ERROR); //= user cancel
             return SyncResult::aborted;
         }
-        const ErrorLog::Stats logCount = errorLog_.getStats();
+        const ErrorLogStats logCount = getStats(errorLog_);
         if (logCount.error > 0)
             return SyncResult::finishedError;
         else if (logCount.warning > 0)
             return SyncResult::finishedWarning;
 
         if (getStatsTotal() == ProgressStats())
-            errorLog_.logMsg(_("Nothing to synchronize"), MSG_TYPE_INFO);
+            logMsg(errorLog_, _("Nothing to synchronize"), MSG_TYPE_INFO);
         return SyncResult::finishedSuccess;
     }();
 
@@ -101,7 +102,7 @@ BatchStatusHandler::Result BatchStatusHandler::reportResults(const Zstring& post
         totalTime
     };
 
-    const AbstractPath logFilePath = generateLogFilePath(logFormat, summary, altLogFolderPathPhrase);
+    AbstractPath logFilePath = AFS::appendRelPath(logFolderPath, generateLogFileName(logFormat, summary));
     //e.g. %AppData%\FreeFileSync\Logs\Backup FreeFileSync 2013-09-15 015052.123 [Error].log
 
     auto notifyStatusNoThrow = [&](std::wstring&& msg) { try { updateStatus(std::move(msg)); /*throw AbortProcess*/ } catch (AbortProcess&) {} };
@@ -139,13 +140,13 @@ BatchStatusHandler::Result BatchStatusHandler::reportResults(const Zstring& post
                                                                             syncResult == SyncResult::finishedError)))
                 try
                 {
+                    logMsg(errorLog_, replaceCpy(_("Sending email notification to %x"), L"%x", utfTo<std::wstring>(notifyEmail)), MSG_TYPE_INFO);
                     sendLogAsEmail(notifyEmail, summary, errorLog_, logFilePath, notifyStatusNoThrow); //throw FileError
-                    errorLog_.logMsg(replaceCpy(_("Sending email notification to %x..."), L"%x", utfTo<std::wstring>(notifyEmail)), MSG_TYPE_INFO);
                 }
-                catch (const FileError& e) { errorLog_.logMsg(e.toString(), MSG_TYPE_ERROR); }
+                catch (const FileError& e) { logMsg(errorLog_, e.toString(), MSG_TYPE_ERROR); }
 
         //--------------------- post sync actions ----------------------
-        auto mayRunAfterCountDown = [&](const std::wstring& operationName)
+        auto proceedWithShutdown = [&](const std::wstring& operationName)
         {
             if (progressDlg_->getWindowIfVisible())
                 try
@@ -160,7 +161,7 @@ BatchStatusHandler::Result BatchStatusHandler::reportResults(const Zstring& post
                                 throw;
                         }
                     };
-                    delayAndCountDown(std::chrono::steady_clock::now() + std::chrono::seconds(5), notifyStatusThrowOnCancel); //throw AbortProcess
+                    delayAndCountDown(std::chrono::steady_clock::now() + std::chrono::seconds(10), notifyStatusThrowOnCancel); //throw AbortProcess
                 }
                 catch (AbortProcess&) { return false; }
 
@@ -176,14 +177,14 @@ BatchStatusHandler::Result BatchStatusHandler::reportResults(const Zstring& post
                 assert(false);
                 break;
             case PostSyncAction2::sleep:
-                if (mayRunAfterCountDown(_("System: Sleep")))
+                if (proceedWithShutdown(_("System: Sleep")))
                 {
                     autoClose = progressDlg_->getOptionAutoCloseDialog();
                     suspend = true;
                 }
                 break;
             case PostSyncAction2::shutdown:
-                if (mayRunAfterCountDown(_("System: Shut down")))
+                if (proceedWithShutdown(_("System: Shut down")))
                 {
                     autoClose = true;
                     finalRequest = FinalRequest::shutdown; //system shutdown must be handled by calling context!
@@ -195,7 +196,7 @@ BatchStatusHandler::Result BatchStatusHandler::reportResults(const Zstring& post
         if (!autoClose) //only play when showing results dialog
             if (!soundFileSyncComplete_.empty())
             {
-                //wxWidgets shows modal error dialog by default => NO!
+                //wxWidgets shows modal error dialog by default => "no, wxWidgets, NO!"
                 wxLog* oldLogTarget = wxLog::SetActiveTarget(new wxLogStderr); //transfer and receive ownership!
                 ZEN_ON_SCOPE_EXIT(delete wxLog::SetActiveTarget(oldLogTarget));
 
@@ -211,16 +212,27 @@ BatchStatusHandler::Result BatchStatusHandler::reportResults(const Zstring& post
         //do NOT use tryReportingError()! saving log files should not be cancellable!
         saveLogFile(logFilePath, summary, errorLog_, logfilesMaxAgeDays, logFormat, logFilePathsToKeep, notifyStatusNoThrow); //throw FileError
     }
-    catch (const FileError& e) { errorLog_.logMsg(e.toString(), MSG_TYPE_ERROR); logFatalError(e.toString()); }
-    //----------------------------------------------------------
+    catch (const FileError& e)
+    {
+        logMsg(errorLog_, e.toString(), MSG_TYPE_ERROR);
 
+        const AbstractPath logFileDefaultPath = AFS::appendRelPath(createAbstractPath(getLogFolderDefaultPath()), generateLogFileName(logFormat, summary));
+        if (logFilePath != logFileDefaultPath) //fallback: log file *must* be saved no matter what!
+            try
+            {
+                logFilePath = logFileDefaultPath;
+                saveLogFile(logFileDefaultPath, summary, errorLog_, logfilesMaxAgeDays, logFormat, logFilePathsToKeep, notifyStatusNoThrow); //throw FileError
+            }
+            catch (const FileError& e2) { logMsg(errorLog_, e2.toString(), MSG_TYPE_ERROR); }
+    }
+    //----------------------------------------------------------
 
     if (suspend) //...*before* results dialog is shown
         try
         {
             suspendSystem(); //throw FileError
         }
-        catch (const FileError& e) { errorLog_.logMsg(e.toString(), MSG_TYPE_ERROR); }
+        catch (const FileError& e) { logMsg(errorLog_, e.toString(), MSG_TYPE_ERROR); }
 
     if (switchToGuiRequested_) //-> avoid recursive yield() calls, thous switch not before ending batch mode
     {
@@ -235,7 +247,13 @@ BatchStatusHandler::Result BatchStatusHandler::reportResults(const Zstring& post
                                                                                   syncResult, errorLogFinal);
     progressDlg_ = nullptr;
 
-    return {syncResult, errorLogFinal.ref().getStats(), finalRequest, logFilePath, dlgSize, dlgIsMaximized};
+    return {syncResult, getStats(errorLogFinal.ref()), finalRequest, logFilePath, dlgSize, dlgIsMaximized};
+}
+
+
+wxWindow* BatchStatusHandler::getWindowIfVisible()
+{
+    return progressDlg_ ? progressDlg_->getWindowIfVisible() : nullptr;
 }
 
 
@@ -259,9 +277,21 @@ void BatchStatusHandler::updateDataProcessed(int itemsDelta, int64_t bytesDelta)
 }
 
 
-void BatchStatusHandler::logInfo(const std::wstring& msg)
+void BatchStatusHandler::logMessage(const std::wstring& msg, MsgType type)
 {
-    errorLog_.logMsg(msg, MSG_TYPE_INFO);
+    logMsg(errorLog_, msg, [&]
+    {
+        switch (type)
+        {
+            //*INDENT-OFF*
+            case MsgType::info:    return MSG_TYPE_INFO;
+            case MsgType::warning: return MSG_TYPE_WARNING;
+            case MsgType::error:   return MSG_TYPE_ERROR;
+            //*INDENT-ON*
+        }
+        assert(false);
+        return MSG_TYPE_ERROR;
+    }());
     requestUiUpdate(false /*force*/); //throw AbortProcess
 }
 
@@ -270,7 +300,7 @@ void BatchStatusHandler::reportWarning(const std::wstring& msg, bool& warningAct
 {
     PauseTimers dummy(*progressDlg_);
 
-    errorLog_.logMsg(msg, MSG_TYPE_WARNING);
+    logMsg(errorLog_, msg, MSG_TYPE_WARNING);
 
     if (!warningActive)
         return;
@@ -285,6 +315,7 @@ void BatchStatusHandler::reportWarning(const std::wstring& msg, bool& warningAct
                 bool dontWarnAgain = false;
                 switch (showQuestionDialog(progressDlg_->getWindowIfVisible(), DialogInfoType::warning,
                                            PopupDialogCfg().setDetailInstructions(msg + L"\n\n" + _("You can switch to FreeFileSync's main window to resolve this issue.")).
+                                           alertWhenPending(soundFileAlertPending_).
                                            setCheckBox(dontWarnAgain, _("&Don't show this warning again"), static_cast<ConfirmationButton3>(QuestionButton2::no)),
                                            _("&Ignore"), _("&Switch")))
                 {
@@ -293,7 +324,7 @@ void BatchStatusHandler::reportWarning(const std::wstring& msg, bool& warningAct
                         break;
 
                     case QuestionButton2::no: //switch
-                        errorLog_.logMsg(_("Switching to FreeFileSync's main window"), MSG_TYPE_INFO);
+                        logMsg(errorLog_, _("Switching to FreeFileSync's main window"), MSG_TYPE_INFO);
                         switchToGuiRequested_ = true; //treat as a special kind of cancel
                         abortProcessNow(AbortTrigger::user); //throw AbortProcess
 
@@ -315,20 +346,23 @@ ProcessCallback::Response BatchStatusHandler::reportError(const ErrorInfo& error
 {
     PauseTimers dummy(*progressDlg_);
 
+    //log actual fail time (not "now"!)
+    const time_t failTime = std::chrono::system_clock::to_time_t(std::chrono::system_clock::now() -
+                                                                 std::chrono::duration_cast<std::chrono::system_clock::duration>(std::chrono::steady_clock::now() - errorInfo.failTime));
     //auto-retry
     if (errorInfo.retryNumber < autoRetryCount_)
     {
-        errorLog_.logMsg(errorInfo.msg + L"\n-> " + _("Automatic retry"), MSG_TYPE_INFO);
+        logMsg(errorLog_, errorInfo.msg + L"\n-> " + _("Automatic retry"), MSG_TYPE_INFO, failTime);
         delayAndCountDown(errorInfo.failTime + autoRetryDelay_,
                           [&, statusPrefix  = _("Automatic retry") +
-                                              (errorInfo.retryNumber == 0 ? L"" : L' ' + numberTo<std::wstring>(errorInfo.retryNumber + 1)) + L" | ",
-                              statusPostfix = L" | " + _("Error") + L": " + replaceCpy(errorInfo.msg, L'\n', L' ')](const std::wstring& timeRemMsg)
+                                              (errorInfo.retryNumber == 0 ? L"" : L' ' + formatNumber(errorInfo.retryNumber + 1)) + SPACED_DASH,
+                              statusPostfix = SPACED_DASH + _("Error") + L": " + replaceCpy(errorInfo.msg, L'\n', L' ')](const std::wstring& timeRemMsg)
         { this->updateStatus(statusPrefix + timeRemMsg + statusPostfix); }); //throw AbortProcess
         return ProcessCallback::retry;
     }
 
     //always, except for "retry":
-    auto guardWriteLog = makeGuard<ScopeGuardRunMode::onExit>([&] { errorLog_.logMsg(errorInfo.msg, MSG_TYPE_ERROR); });
+    auto guardWriteLog = makeGuard<ScopeGuardRunMode::onExit>([&] { logMsg(errorLog_, errorInfo.msg, MSG_TYPE_ERROR, failTime); });
 
     if (!progressDlg_->getOptionIgnoreErrors())
     {
@@ -339,7 +373,8 @@ ProcessCallback::Response BatchStatusHandler::reportError(const ErrorInfo& error
                 forceUiUpdateNoThrow(); //noexcept! => don't throw here when error occurs during clean up!
 
                 switch (showConfirmationDialog(progressDlg_->getWindowIfVisible(), DialogInfoType::error,
-                                               PopupDialogCfg().setDetailInstructions(errorInfo.msg),
+                                               PopupDialogCfg().setDetailInstructions(errorInfo.msg).
+                                               alertWhenPending(soundFileAlertPending_),
                                                _("&Ignore"), _("Ignore &all"), _("&Retry")))
                 {
                     case ConfirmationButton3::accept: //ignore
@@ -351,7 +386,7 @@ ProcessCallback::Response BatchStatusHandler::reportError(const ErrorInfo& error
 
                     case ConfirmationButton3::decline: //retry
                         guardWriteLog.dismiss();
-                        errorLog_.logMsg(errorInfo.msg + L"\n-> " + _("Retrying operation..."), MSG_TYPE_INFO);
+                        logMsg(errorLog_, errorInfo.msg + L"\n-> " + _("Retrying operation..."), MSG_TYPE_INFO, failTime);
                         return ProcessCallback::retry;
 
                     case ConfirmationButton3::cancel:
@@ -378,7 +413,7 @@ void BatchStatusHandler::reportFatalError(const std::wstring& msg)
 {
     PauseTimers dummy(*progressDlg_);
 
-    errorLog_.logMsg(msg, MSG_TYPE_ERROR);
+    logMsg(errorLog_, msg, MSG_TYPE_ERROR);
 
     if (!progressDlg_->getOptionIgnoreErrors())
         switch (batchErrorHandling_)
@@ -388,8 +423,8 @@ void BatchStatusHandler::reportFatalError(const std::wstring& msg)
                 forceUiUpdateNoThrow(); //noexcept! => don't throw here when error occurs during clean up!
 
                 switch (showConfirmationDialog(progressDlg_->getWindowIfVisible(), DialogInfoType::error,
-                                               PopupDialogCfg().setTitle(_("Error")).
-                                               setDetailInstructions(msg),
+                                               PopupDialogCfg().setDetailInstructions(msg).
+                                               alertWhenPending(soundFileAlertPending_),
                                                _("&Ignore"), _("Ignore &all")))
                 {
                     case ConfirmationButton2::accept: //ignore
