@@ -5,12 +5,12 @@
 // *****************************************************************************
 
 #include "log_panel.h"
-#include <wx/clipbrd.h>
 #include <wx+/window_tools.h>
 #include <wx+/image_resources.h>
 #include <wx+/rtl.h>
 #include <wx+/context_menu.h>
 #include <wx+/popup_dlg.h>
+//#include <wx+/std_button_layout.h>
 
 using namespace zen;
 using namespace fff;
@@ -150,7 +150,7 @@ namespace
 class GridDataMessages : public GridData
 {
 public:
-    GridDataMessages(const SharedRef<const ErrorLog>& log) : msgView_(log) {}
+    explicit GridDataMessages(const SharedRef<const ErrorLog>& log) : msgView_(log) {}
 
     MessageView& getDataView() { return msgView_; }
 
@@ -163,7 +163,7 @@ public:
             {
                 case ColumnTypeLog::time:
                     if (entry->firstLine)
-                        return utfTo<std::wstring>(formatTime(formatTimeTag, getLocalTime(entry->time)));
+                        return utfTo<std::wstring>(formatTime(formatTimeTag, getLocalTime(entry->time))); //empty string on error
                     break;
 
                 case ColumnTypeLog::severity:
@@ -344,11 +344,11 @@ void LogPanel::setLog(const std::shared_ptr<const ErrorLog>& log)
             return SharedRef<const ErrorLog>(log);
 
         ErrorLog dummyLog;
-        dummyLog.logMsg(_("No log entries"), MSG_TYPE_INFO);
+        logMsg(dummyLog, _("No log entries"), MSG_TYPE_INFO);
         return makeSharedRef<const ErrorLog>(std::move(dummyLog));
     }();
 
-    const ErrorLog::Stats logCount = newLog.ref().getStats();
+    const ErrorLogStats logCount = getStats(newLog.ref());
 
     auto initButton = [](ToggleButton& btn, const char* imgName, const wxString& tooltip)
     {
@@ -419,6 +419,27 @@ void LogPanel::onInfo(wxCommandEvent& event)
 }
 
 
+void LogPanel::onMsgGridContext(GridContextMenuEvent& event)
+{
+    const std::vector<size_t> selection = m_gridMessages->getSelectedRows();
+
+    const size_t rowCount = [&]() -> size_t
+    {
+        if (auto prov = m_gridMessages->getDataProvider())
+            return prov->getRowCount();
+        return 0;
+    }();
+
+    ContextMenu menu;
+    menu.addItem(_("&Copy") + L"\tCtrl+C", [this] { copySelectionToClipboard(); }, loadImage("item_copy_sicon"), !selection.empty());
+    menu.addSeparator();
+
+    menu.addItem(_("Select all") + L"\tCtrl+A", [this] { m_gridMessages->selectAllRows(GridEventPolicy::allow); }, wxNullImage, rowCount > 0);
+
+    menu.popup(*m_gridMessages, event.mousePos_);
+}
+
+
 void LogPanel::onGridButtonEvent(wxKeyEvent& event)
 {
     int keyCode = event.GetKeyCode();
@@ -426,10 +447,9 @@ void LogPanel::onGridButtonEvent(wxKeyEvent& event)
     if (event.ControlDown())
         switch (keyCode)
         {
-            //case 'A': -> "select all" is already implemented by Grid!
-
             case 'C':
             case WXK_INSERT: //CTRL + C || CTRL + INS
+            case WXK_NUMPAD_INSERT:
                 copySelectionToClipboard();
                 return; // -> swallow event! don't allow default grid commands!
         }
@@ -446,84 +466,59 @@ void LogPanel::onGridButtonEvent(wxKeyEvent& event)
 }
 
 
-void LogPanel::onMsgGridContext(GridContextMenuEvent& event)
-{
-    const std::vector<size_t> selection = m_gridMessages->getSelectedRows();
-
-    const size_t rowCount = [&]() -> size_t
-    {
-        if (auto prov = m_gridMessages->getDataProvider())
-            return prov->getRowCount();
-        return 0;
-    }();
-
-    ContextMenu menu;
-    menu.addItem(_("Copy") + L"\tCtrl+C", [this] { copySelectionToClipboard(); }, wxNullImage, !selection.empty());
-    menu.addSeparator();
-
-    menu.addItem(_("Select all") + L"\tCtrl+A", [this] { m_gridMessages->selectAllRows(GridEventPolicy::allow); }, wxNullImage, rowCount > 0);
-    menu.popup(*m_gridMessages);
-}
-
-
 void LogPanel::onLocalKeyEvent(wxKeyEvent& event) //process key events without explicit menu entry :)
 {
-    if (processingKeyEventHandler_) //avoid recursion
+    if (!processingKeyEventHandler_) //avoid recursion
     {
-        event.Skip();
-        return;
-    }
-    processingKeyEventHandler_ = true;
-    ZEN_ON_SCOPE_EXIT(processingKeyEventHandler_ = false);
+        processingKeyEventHandler_ = true;
+        ZEN_ON_SCOPE_EXIT(processingKeyEventHandler_ = false);
 
+        const int keyCode = event.GetKeyCode();
 
-    const int keyCode = event.GetKeyCode();
+        if (event.ControlDown())
+            switch (keyCode)
+            {
+                case 'A':
+                    m_gridMessages->SetFocus();
+                    m_gridMessages->selectAllRows(GridEventPolicy::allow);
+                    return; // -> swallow event! don't allow default grid commands!
 
-    if (event.ControlDown())
-        switch (keyCode)
-        {
-            case 'A':
-                m_gridMessages->SetFocus();
-                m_gridMessages->selectAllRows(GridEventPolicy::allow);
-                return; // -> swallow event! don't allow default grid commands!
+                    //case 'C': -> already implemented by "Grid" class
+            }
+        else
+            switch (keyCode)
+            {
+                //redirect certain (unhandled) keys directly to grid!
+                case WXK_UP:
+                case WXK_DOWN:
+                case WXK_LEFT:
+                case WXK_RIGHT:
+                case WXK_PAGEUP:
+                case WXK_PAGEDOWN:
+                case WXK_HOME:
+                case WXK_END:
 
-                //case 'C': -> already implemented by "Grid" class
-        }
-    else
-        switch (keyCode)
-        {
-            //redirect certain (unhandled) keys directly to grid!
-            case WXK_UP:
-            case WXK_DOWN:
-            case WXK_LEFT:
-            case WXK_RIGHT:
-            case WXK_PAGEUP:
-            case WXK_PAGEDOWN:
-            case WXK_HOME:
-            case WXK_END:
-
-            case WXK_NUMPAD_UP:
-            case WXK_NUMPAD_DOWN:
-            case WXK_NUMPAD_LEFT:
-            case WXK_NUMPAD_RIGHT:
-            case WXK_NUMPAD_PAGEUP:
-            case WXK_NUMPAD_PAGEDOWN:
-            case WXK_NUMPAD_HOME:
-            case WXK_NUMPAD_END:
-                if (!isComponentOf(wxWindow::FindFocus(), m_gridMessages) && //don't propagate keyboard commands if grid is already in focus
-                    m_gridMessages->IsEnabled())
-                    if (wxEvtHandler* evtHandler = m_gridMessages->getMainWin().GetEventHandler())
+                case WXK_NUMPAD_UP:
+                case WXK_NUMPAD_DOWN:
+                case WXK_NUMPAD_LEFT:
+                case WXK_NUMPAD_RIGHT:
+                case WXK_NUMPAD_PAGEUP:
+                case WXK_NUMPAD_PAGEDOWN:
+                case WXK_NUMPAD_HOME:
+                case WXK_NUMPAD_END:
+                    if (!isComponentOf(wxWindow::FindFocus(), m_gridMessages) && //don't propagate keyboard commands if grid is already in focus
+                        m_gridMessages->IsEnabled())
                     {
                         m_gridMessages->SetFocus();
 
                         event.SetEventType(wxEVT_KEY_DOWN); //the grid event handler doesn't expect wxEVT_CHAR_HOOK!
-                        evtHandler->ProcessEvent(event); //propagating event catched at wxTheApp to child leads to recursion, but we prevented it...
+                        m_gridMessages->getMainWin().GetEventHandler()->ProcessEvent(event); //propagating event catched at wxTheApp to child leads to recursion, but we prevented it...
                         event.Skip(false); //definitively handled now!
                         return;
                     }
-                break;
-        }
-
+                    break;
+            }
+    }
     event.Skip();
 }
 
@@ -532,7 +527,7 @@ void LogPanel::copySelectionToClipboard()
 {
     try
     {
-        std::wstring clipBuf; //guaranteed exponential growth, unlike wxString
+        std::wstring clipBuf; //perf: wxString doesn't model exponential growth => unsuitable for large data sets
 
         if (auto prov = m_gridMessages->getDataProvider())
         {
@@ -552,13 +547,7 @@ void LogPanel::copySelectionToClipboard()
                 }
         }
 
-        //finally write to clipboard
-        if (!clipBuf.empty())
-            if (wxClipboard::Get()->Open())
-            {
-                ZEN_ON_SCOPE_EXIT(wxClipboard::Get()->Close());
-                wxClipboard::Get()->SetData(new wxTextDataObject(std::move(clipBuf))); //ownership passed
-            }
+        setClipboardText(clipBuf);
     }
     catch (const std::bad_alloc& e)
     {
