@@ -23,6 +23,7 @@
 #include "../base/algorithm.h"
 #include "../return_codes.h"
 
+
 namespace fff
 {
 class FolderPairFirst;
@@ -46,15 +47,15 @@ public:
                        const std::vector<Zstring>& referenceFiles,
                        bool startComparison);
 
-    void onQueryEndSession(); //last chance to do something useful before killing the application!
 private:
     MainDialog(const Zstring& globalConfigFilePath,
                const XmlGuiConfig& guiCfg,
                const std::vector<Zstring>& referenceFiles,
-               const XmlGlobalSettings& globalSettings, //take over ownership => save on exit
-               bool startComparison);
+               const XmlGlobalSettings& globalSettings); //take over ownership => save on exit
     ~MainDialog();
 
+    void onBeforeSystemShutdown(); //last chance to do something useful before killing the application!
+
     friend class StatusHandlerTemporaryPanel;
     friend class StatusHandlerFloatingDialog;
     friend class FolderPairFirst;
@@ -76,11 +77,11 @@ private:
     void setGlobalCfgOnInit(const XmlGlobalSettings& globalSettings); //messes with Maximize(), window sizes, so call just once!
     XmlGlobalSettings getGlobalCfgBeforeExit(); //destructive "get" thanks to "Iconize(false), Maximize(false)"
 
-    bool loadConfiguration(const std::vector<Zstring>& filepaths); //return "true" if loaded successfully; "false" if cancelled or error
+    bool loadConfiguration(const std::vector<Zstring>& filepaths, bool ignoreBrokenConfig = false); //"false": error/cancel
 
-    bool trySaveConfig     (const Zstring* guiCfgPath); //return true if saved successfully
-    bool trySaveBatchConfig(const Zstring* batchCfgPath); //
-    bool saveOldConfig(); //return false on user abort
+    bool trySaveConfig     (const Zstring* guiCfgPath);   //
+    bool trySaveBatchConfig(const Zstring* batchCfgPath); //"false": error/cancel
+    bool saveOldConfig();                                 //
 
     void updateGlobalFilterButton();
 
@@ -104,29 +105,33 @@ private:
     void updateStatistics();       // more fine-grained updaters
     void updateUnsavedCfgStatus(); //
 
+    std::vector<std::wstring> getJobNames() const;
+
     //context menu functions
     std::vector<FileSystemObject*> getGridSelection(bool fromLeft = true, bool fromRight = true) const;
     std::vector<FileSystemObject*> getTreeSelection() const;
 
     void setSyncDirManually (const std::vector<FileSystemObject*>& selection, SyncDirection direction);
     void setIncludedManually(const std::vector<FileSystemObject*>& selection, bool setIncluded);
-    void copySelectionToClipboard(const std::vector<const zen::Grid*>& gridRefs);
+    void copyGridSelectionToClipboard(const zen::Grid& grid);
+    void copyPathsToClipboard(const std::vector<FileSystemObject*>& selectionL,
+                              const std::vector<FileSystemObject*>& selectionR);
 
-    void copyToAlternateFolder(const std::vector<FileSystemObject*>& selectionLeft,
-                               const std::vector<FileSystemObject*>& selectionRight);
+    void copyToAlternateFolder(const std::vector<FileSystemObject*>& selectionL,
+                               const std::vector<FileSystemObject*>& selectionR);
 
-    void deleteSelectedFiles(const std::vector<FileSystemObject*>& selectionLeft,
-                             const std::vector<FileSystemObject*>& selectionRight, bool moveToRecycler);
+    void deleteSelectedFiles(const std::vector<FileSystemObject*>& selectionL,
+                             const std::vector<FileSystemObject*>& selectionR, bool moveToRecycler);
 
     void openExternalApplication(const Zstring& commandLinePhrase, bool leftSide,
-                                 const std::vector<FileSystemObject*>& selectionLeft,
-                                 const std::vector<FileSystemObject*>& selectionRight); //selection may be empty
+                                 const std::vector<FileSystemObject*>& selectionL,
+                                 const std::vector<FileSystemObject*>& selectionR); //selection may be empty
 
-    //status bar supports one of the following two states at a time:
     void setStatusBarFileStats(FileView::FileStats statsLeft, FileView::FileStats statsRight);
-    //void setStatusBarFullText(const wxString& msg);
 
-    void flashStatusInformation(const wxString& msg); //temporarily show different status (only valid for setStatusBarFileStats)
+    void setStatusInfo(const wxString& text, bool highlight); //(permanently) set status bar center text
+    void flashStatusInfo(const wxString& text); //temporarily show different status
+    void popStatusInfo();
 
     //events
     void onGridKeyEvent(wxKeyEvent& event, zen::Grid& grid, bool leftSide);
@@ -143,8 +148,8 @@ private:
     void onGridGroupContextRim(zen::GridClickEvent& event, bool leftSide);
 
     void onGridContextRim(const std::vector<FileSystemObject*>& selection,
-                          const std::vector<FileSystemObject*>& selectionLeft,
-                          const std::vector<FileSystemObject*>& selectionRight, bool leftSide);
+                          const std::vector<FileSystemObject*>& selectionL,
+                          const std::vector<FileSystemObject*>& selectionR, bool leftSide, wxPoint mousePos);
 
     void onTreeGridContext(zen::GridContextMenuEvent& event);
 
@@ -158,6 +163,8 @@ private:
     void onCheckRows       (CheckRowsEvent&     event);
     void onSetSyncDirection(SyncDirectionEvent& event);
 
+    void swapSides();
+
     void onGridDoubleClickRim(zen::GridClickEvent& event, bool leftSide);
 
     void onGridLabelLeftClickRim(zen::GridLabelClickEvent& event, bool onLeft);
@@ -200,8 +207,8 @@ private:
     void onToggleLog            (wxCommandEvent& event) override;
     void onCompare              (wxCommandEvent& event) override;
     void onStartSync            (wxCommandEvent& event) override;
-    void onSwapSides            (wxCommandEvent& event) override;
     void onClose                (wxCloseEvent&   event) override;
+    void onSwapSides            (wxCommandEvent& event) override { swapSides(); }
 
     void startSyncForSelecction(const std::vector<FileSystemObject*>& selection);
 
@@ -301,6 +308,7 @@ private:
 
     //the prime data structure of this tool *bling*:
     FolderComparison folderCmp_; //optional!: sync button not available if empty
+    std::shared_ptr<const zen::ErrorLog> errorLogCmp_;
 
     //folder pairs:
     std::unique_ptr<FolderPairFirst> firstFolderPair_; //always bound!!!
@@ -316,8 +324,9 @@ private:
     //-------------------------------------
 
     //***********************************************
-    //status information
-    std::vector<wxString> oldStatusMsgs_; //the first one is the original/non-flash status message
+    //status bar center text
+    std::vector<wxString> statusTxts_; //the first one is the original/non-flash status message
+    bool statusTxtHighlightFirst_ = false;
 
     //compare status panel (hidden on start, shown when comparing)
     std::unique_ptr<CompareProgressPanel> compareStatus_; //always bound
@@ -344,8 +353,6 @@ private:
 
     zen::AsyncGuiQueue guiQueue_; //schedule and run long-running tasks asynchronously, but process results on GUI queue
 
-    std::unique_ptr<FilterConfig> filterCfgOnClipboard_; //copy/paste of filter config
-
     wxWindowID focusAfterCloseLog_    = wxID_ANY; //
     wxWindowID focusAfterCloseSearch_ = wxID_ANY; //restore focus after panel is closed
     //don't save wxWindow* to arbitrary window: might not exist anymore when hideFindPanel() uses it!!! (e.g. some folder pair panel)
@@ -357,6 +364,9 @@ private:
     TempFileBuffer tempFileBuf_; //buffer temporary copies of non-native files for %local_path%
 
     const wxImage imgTrashSmall_;
+    const wxImage imgFileManagerSmall_;
+
+    const zen::SharedRef<std::function<void()>> onBeforeSystemShutdownCookie_ = zen::makeSharedRef<std::function<void()>>([this] { onBeforeSystemShutdown(); });
 };
 }
 
