@@ -136,24 +136,25 @@ void TreeView::extractVisibleSubtree(ContainerObject& hierObj,  //in
 
 namespace
 {
-//generate nice percentage numbers which precisely sum up to 100
+//generate nice percentage numbers which precisely add up to 100
 void calcPercentage(std::vector<std::pair<uint64_t, int*>>& workList)
 {
-    const uint64_t total = std::accumulate(workList.begin(), workList.end(), uint64_t(),
-    [](uint64_t sum, const std::pair<uint64_t, int*>& pair) { return sum + pair.first; });
+    uint64_t bytesTotal = 0;
+    for (const auto& [bytes, percentOut] : workList)
+        bytesTotal += bytes;
 
-    if (total == 0U) //this case doesn't work with the error minimizing algorithm below
+    if (bytesTotal == 0U) //this case doesn't work with the error minimizing algorithm below
     {
-        for (auto& [bytes, percent] : workList)
-            *percent = 0;
+        for (auto& [bytes, percentOut] : workList)
+            *percentOut = 0;
         return;
     }
 
     int remainingPercent = 100;
-    for (auto& [bytes, percent] : workList)
+    for (auto& [bytes, percentOut] : workList)
     {
-        *percent = static_cast<int>(bytes * 100U / total); //round down
-        remainingPercent -= *percent;
+        *percentOut = static_cast<int>(bytes * 100U / bytesTotal); //round down
+        remainingPercent -= *percentOut;
     }
     assert(remainingPercent >= 0);
     assert(remainingPercent < std::ssize(workList));
@@ -163,9 +164,9 @@ void calcPercentage(std::vector<std::pair<uint64_t, int*>>& workList)
     if (remainingPercent > 0)
     {
         std::nth_element(workList.begin(), workList.begin() + remainingPercent - 1, workList.end(),
-                         [total](const std::pair<uint64_t, int*>& lhs, const std::pair<uint64_t, int*>& rhs)
+                         [bytesTotal](const std::pair<uint64_t, int*>& lhs, const std::pair<uint64_t, int*>& rhs)
         {
-            return lhs.first * 100U % total > rhs.first * 100U % total;
+            return lhs.first * 100U % bytesTotal > rhs.first * 100U % bytesTotal;
         });
 
         std::for_each(workList.begin(), workList.begin() + remainingPercent, [&](std::pair<uint64_t, int*>& pair) { ++*pair.second; });
@@ -686,7 +687,7 @@ class GridDataTree : private wxEvtHandler, public GridData
 {
 public:
     GridDataTree(Grid& grid) :
-        widthNodeIcon_(IconBuffer::getSize(IconBuffer::SIZE_SMALL)),
+        widthNodeIcon_(IconBuffer::getSize(IconBuffer::IconSize::small)),
         widthLevelStep_(widthNodeIcon_),
         widthNodeStatus_(loadImage("node_expanded").GetWidth()),
         rootIcon_(loadImage("root_folder", widthNodeIcon_)),
@@ -730,7 +731,7 @@ private:
                             return dirRight;
                         else if (dirRight.empty())
                             return dirLeft;
-                        return dirLeft + L' ' + EN_DASH + L'\n' + dirRight;
+                        return dirLeft + /*L' ' + EM_DASH + */ L'\n' + dirRight;
                     }
                 break;
 
@@ -850,7 +851,7 @@ private:
                     {
                         wxRect areaPerc(rectTmp.x, rectTmp.y + fastFromDIP(2), percentageBarWidth_, rectTmp.height - fastFromDIP(4));
                         //clear background
-                        drawFilledRectangle(dc, areaPerc, fastFromDIP(1), getColorPercentBorder(), getColorPercentBackground());
+                        drawInsetRectangle(dc, areaPerc, fastFromDIP(1), getColorPercentBorder(), getColorPercentBackground());
                         areaPerc.Deflate(fastFromDIP(1));
 
                         //inner area
@@ -1128,7 +1129,7 @@ private:
             setShowPercentage(overviewPanelShowPercentageDefault);
             grid_.setColumnConfig(convertColAttributes(getOverviewDefaultColAttribs(), getOverviewDefaultColAttribs()));
         };
-        menu.addItem(_("&Default"), setDefaultColumns); //'&' -> reuse text from "default" buttons elsewhere
+        menu.addItem(_("&Default"), setDefaultColumns, loadImage("reset_sicon")); //'&' -> reuse text from "default" buttons elsewhere
         //--------------------------------------------------------------------------------------------------------
 
         menu.popup(grid_, {event.mousePos_.x, grid_.getColumnLabelHeight()});
@@ -1145,8 +1146,8 @@ private:
             sortAscending = !ascending;
 
         getDataView().setSortDirection(colTypeTree, sortAscending);
+        grid_.Refresh(); //just in case, but setSortDirection() should not change grid size
         grid_.clearSelection(GridEventPolicy::allow);
-        grid_.Refresh();
     }
 
     void expandNode(size_t row)
@@ -1169,8 +1170,8 @@ private:
     const int gapSize_            = fastFromDIP(TREE_GRID_GAP_SIZE_DIP);
     const int percentageBarWidth_ = fastFromDIP(PERCENTAGE_BAR_WIDTH_DIP);
 
-    const wxImage fileIcon_ = IconBuffer::genericFileIcon(IconBuffer::SIZE_SMALL);
-    const wxImage dirIcon_  = IconBuffer::genericDirIcon (IconBuffer::SIZE_SMALL);
+    const wxImage fileIcon_ = IconBuffer::genericFileIcon(IconBuffer::IconSize::small);
+    const wxImage dirIcon_  = IconBuffer::genericDirIcon (IconBuffer::IconSize::small);
 
     const int widthNodeIcon_;
     const int widthLevelStep_;
@@ -1189,7 +1190,7 @@ void treegrid::init(Grid& grid)
     grid.setDataProvider(std::make_shared<GridDataTree>(grid));
     grid.showRowLabel(false);
 
-    const int rowHeight = std::max(IconBuffer::getSize(IconBuffer::SIZE_SMALL) + 2, //1 extra pixel on top/bottom; dearly needed on OS X!
+    const int rowHeight = std::max(IconBuffer::getSize(IconBuffer::IconSize::small) + 2, //1 extra pixel on top/bottom; dearly needed on OS X!
                                    grid.getMainWin().GetCharHeight()); //seems to already include 3 margin pixels on top/bottom (consider percentage area)
     grid.setRowHeight(rowHeight);
 }
