@@ -8,15 +8,17 @@
 #define FMT_UNIT_8702184019487324
 
 #include <string>
-#include <cstdint>
-#include "string_tools.h"
+#include <optional>
+//#include <cstdint>
+//#include "string_tools.h"
 
 
 namespace zen
 {
+    const int bytesPerKilo = 1000;
 std::wstring formatFilesizeShort(int64_t filesize);
 std::wstring formatRemainingTime(double timeInSec);
-std::wstring formatFraction(double fraction); //within [0, 1]
+std::wstring formatProgressPercent(double fraction /*[0, 1]*/, int decPlaces = 0 /*[0, 9]*/); //rounded down!
 std::wstring formatUtcToLocalTime(time_t utcTime); //like Windows Explorer would...
 
 std::wstring formatTwoDigitPrecision  (double value); //format with fixed number of digits
