@@ -37,7 +37,7 @@ double zen::nextNiceNumber(double blockSize) //round to next number which is a c
 
     //have a look at leading two digits: "nice" numbers start with 1, 2, 2.5 and 5
     const double steps[] = {1, 2, 2.5, 5, 10};
-    return e * numeric::nearMatch(a, std::begin(steps), std::end(steps));
+    return e * numeric::roundToGrid(a, std::begin(steps), std::end(steps));
 }
 
 
@@ -232,7 +232,7 @@ void drawCornerText(wxDC& dc, const wxRect& graphArea, const wxString& txt, Grap
 
 //calculate intersection of polygon with half-plane
 template <class Function, class Function2>
-void cutPoints(std::vector<CurvePoint>& curvePoints, std::vector<char>& oobMarker, Function isInside, Function2 getIntersection, bool doPolygonCut)
+void cutPoints(std::vector<CurvePoint>& curvePoints, std::vector<unsigned char>& oobMarker, Function isInside, Function2 getIntersection, bool doPolygonCut)
 {
     assert(curvePoints.size() == oobMarker.size());
 
@@ -240,8 +240,8 @@ void cutPoints(std::vector<CurvePoint>& curvePoints, std::vector<char>& oobMarke
 
     auto isMarkedOob = [&](size_t index) { return oobMarker[index] != 0; }; //test if point is start of an OOB line
 
-    std::vector<CurvePoint> curvePointsTmp;
-    std::vector<char>       oobMarkerTmp;
+    std::vector<CurvePoint>    curvePointsTmp;
+    std::vector<unsigned char> oobMarkerTmp;
     curvePointsTmp.reserve(curvePoints.size()); //allocating memory for these containers is one
     oobMarkerTmp  .reserve(oobMarker  .size()); //of the more expensive operations of Graph2D!
 
@@ -282,7 +282,8 @@ void cutPoints(std::vector<CurvePoint>& curvePoints, std::vector<char>& oobMarke
 
 struct GetIntersectionX
 {
-    GetIntersectionX(double x) : x_(x) {}
+    explicit GetIntersectionX(double x) : x_(x) {}
+
     CurvePoint operator()(const CurvePoint& from, const CurvePoint& to) const
     {
         const double deltaX = to.x - from.x;
@@ -296,7 +297,8 @@ private:
 
 struct GetIntersectionY
 {
-    GetIntersectionY(double y) : y_(y) {}
+    explicit GetIntersectionY(double y) : y_(y) {}
+
     CurvePoint operator()(const CurvePoint& from, const CurvePoint& to) const
     {
         const double deltaX = to.x - from.x;
@@ -308,13 +310,13 @@ private:
     const double y_;
 };
 
-void cutPointsOutsideX(std::vector<CurvePoint>& curvePoints, std::vector<char>& oobMarker, double minX, double maxX, bool doPolygonCut)
+void cutPointsOutsideX(std::vector<CurvePoint>& curvePoints, std::vector<unsigned char>& oobMarker, double minX, double maxX, bool doPolygonCut)
 {
     cutPoints(curvePoints, oobMarker, [&](const CurvePoint& pt) { return pt.x >= minX; }, GetIntersectionX(minX), doPolygonCut);
     cutPoints(curvePoints, oobMarker, [&](const CurvePoint& pt) { return pt.x <= maxX; }, GetIntersectionX(maxX), doPolygonCut);
 }
 
-void cutPointsOutsideY(std::vector<CurvePoint>& curvePoints, std::vector<char>& oobMarker, double minY, double maxY, bool doPolygonCut)
+void cutPointsOutsideY(std::vector<CurvePoint>& curvePoints, std::vector<unsigned char>& oobMarker, double minY, double maxY, bool doPolygonCut)
 {
     cutPoints(curvePoints, oobMarker, [&](const CurvePoint& pt) { return pt.y >= minY; }, GetIntersectionY(minY), doPolygonCut);
     cutPoints(curvePoints, oobMarker, [&](const CurvePoint& pt) { return pt.y <= maxY; }, GetIntersectionY(maxY), doPolygonCut);
@@ -491,9 +493,7 @@ void Graph2D::onMouseLeftUp(wxMouseEvent& event)
     {
         if (activeSel_->getStartPos() != activeSel_->refCurrentPos()) //if it's just a single mouse click: discard selection
         {
-            GraphSelectEvent selEvent(activeSel_->refSelection()); //fire off GraphSelectEvent
-            if (wxEvtHandler* handler = GetEventHandler())
-                handler->AddPendingEvent(selEvent);
+            GetEventHandler()->AddPendingEvent(GraphSelectEvent(activeSel_->refSelection()));
 
             oldSel_.push_back(activeSel_->refSelection()); //commit selection
         }
@@ -524,7 +524,7 @@ void Graph2D::addCurve(const SharedRef<CurveData>& data, const CurveAttributes&
 void Graph2D::render(wxDC& dc) const
 {
     //set label font right at the start so that it is considered by wxDC::GetTextExtent() below!
-    dc.SetFont(labelFont_);
+    dc.SetFont(GetFont());
 
     const wxRect clientRect = GetClientRect(); //DON'T use wxDC::GetSize()! DC may be larger than visible area!
 
@@ -576,7 +576,7 @@ void Graph2D::render(wxDC& dc) const
     assert(attr_.yLabelpos == YLabelPos::none || attr_.labelFmtY);
 
     //paint graph background (excluding label area)
-    drawFilledRectangle(dc, graphArea, fastFromDIP(1), getBorderColor(), attr_.colorBack);
+    drawInsetRectangle(dc, graphArea, fastFromDIP(1), getBorderColor(), attr_.colorBack);
     graphArea.Deflate(fastFromDIP(1));
 
     //set label areas respecting graph area border!
@@ -588,16 +588,16 @@ void Graph2D::render(wxDC& dc) const
     double minX = attr_.minX ? *attr_.minX :  std::numeric_limits<double>::infinity(); //automatic: ensure values are initialized by first curve
     double maxX = attr_.maxX ? *attr_.maxX : -std::numeric_limits<double>::infinity(); //
     for (const auto& [curve, attrib] : curves_)
-        {
-            const std::pair<double, double> rangeX = curve.ref().getRangeX();
-            assert(rangeX.first <= rangeX.second + 1.0e-9);
-            //GCC fucks up badly when comparing two *binary identical* doubles and finds "begin > end" with diff of 1e-18
+    {
+        const std::pair<double, double> rangeX = curve.ref().getRangeX();
+        assert(rangeX.first <= rangeX.second + 1.0e-9);
+        //GCC fucks up badly when comparing two *binary identical* doubles and finds "begin > end" with diff of 1e-18
 
-            if (!attr_.minX)
-                minX = std::min(minX, rangeX.first);
-            if (!attr_.maxX)
-                maxX = std::max(maxX, rangeX.second);
-        }
+        if (!attr_.minX)
+            minX = std::min(minX, rangeX.first);
+        if (!attr_.maxX)
+            maxX = std::max(maxX, rangeX.second);
+    }
 
     if (minX <= maxX && maxX - minX < std::numeric_limits<double>::infinity()) //valid x-range
     {
@@ -615,33 +615,33 @@ void Graph2D::render(wxDC& dc) const
         double minY = attr_.minY ? *attr_.minY :  std::numeric_limits<double>::infinity(); //automatic: ensure values are initialized by first curve
         double maxY = attr_.maxY ? *attr_.maxY : -std::numeric_limits<double>::infinity(); //
 
-        std::vector<std::vector<CurvePoint>> curvePoints(curves_.size());
-        std::vector<std::vector<char>>       oobMarker  (curves_.size()); //effectively a std::vector<bool> marking points that start an out-of-bounds line
+        std::vector<std::vector<CurvePoint>>    curvePoints(curves_.size());
+        std::vector<std::vector<unsigned char>> oobMarker  (curves_.size()); //effectively a std::vector<bool> marking points that start an out-of-bounds line
 
         for (size_t index = 0; index < curves_.size(); ++index)
+        {
+            const CurveData&         curve  = curves_    [index].first.ref();
+            std::vector<CurvePoint>& points = curvePoints[index];
+            auto&                    marker = oobMarker  [index];
+
+            points = curve.getPoints(minX, maxX, graphArea.GetSize());
+            marker.resize(points.size()); //default value: false
+            if (!points.empty())
             {
-                const CurveData&         curve  = curves_    [index].first.ref();
-                std::vector<CurvePoint>& points = curvePoints[index];
-                auto&                    marker = oobMarker  [index];
+                //cut points outside visible x-range now in order to calculate height of visible line fragments only!
+                const bool doPolygonCut = curves_[index].second.fillMode == CurveFillMode::polygon; //impacts auto minY/maxY!!
+                cutPointsOutsideX(points, marker, minX, maxX, doPolygonCut);
 
-                points = curve.getPoints(minX, maxX, graphArea.GetSize());
-                marker.resize(points.size()); //default value: false
-                if (!points.empty())
+                if (!attr_.minY || !attr_.maxY)
                 {
-                    //cut points outside visible x-range now in order to calculate height of visible line fragments only!
-                    const bool doPolygonCut = curves_[index].second.fillMode == CurveFillMode::polygon; //impacts auto minY/maxY!!
-                    cutPointsOutsideX(points, marker, minX, maxX, doPolygonCut);
-
-                    if (!attr_.minY || !attr_.maxY)
-                    {
-                        auto itPair = std::minmax_element(points.begin(), points.end(), [](const CurvePoint& lhs, const CurvePoint& rhs) { return lhs.y < rhs.y; });
-                        if (!attr_.minY)
-                            minY = std::min(minY, itPair.first->y);
-                        if (!attr_.maxY)
-                            maxY = std::max(maxY, itPair.second->y);
-                    }
+                    const auto& [itMin, itMax] = std::minmax_element(points.begin(), points.end(), [](const CurvePoint& lhs, const CurvePoint& rhs) { return lhs.y < rhs.y; });
+                    if (!attr_.minY)
+                        minY = std::min(minY, itMin->y);
+                    if (!attr_.maxY)
+                        maxY = std::max(maxY, itMax->y);
                 }
             }
+        }
 
         if (minY <= maxY) //valid y-range
         {
@@ -730,7 +730,7 @@ void Graph2D::render(wxDC& dc) const
                         //unlike wxDC::DrawRectangle() which just widens inner area!
                         wxDCPenChanger   dummy (dc, wxPen(it->second.fillColor, 1 /*[!] width*/));
                         wxDCBrushChanger dummy2(dc, it->second.fillColor);
-                        dc.DrawPolygon(static_cast<int>(points.size()), &points[0]);
+                        dc.DrawPolygon(static_cast<int>(points.size()), points.data());
                     }
 
             //2. draw all currently set mouse selections (including active selection)
