@@ -32,6 +32,20 @@ void setStandardButtonLayout(wxBoxSizer& sizer, const StdButtons& buttons = StdB
 //sizer width will change! => call wxWindow::Fit and wxWindow::Layout
 
 
+inline
+int getDefaultMenuIconSize()
+{
+    return fastFromDIP(20);
+}
+
+
+inline
+int getDefaultButtonHeight()
+{
+    const int defaultHeight = wxButton::GetDefaultSize().GetHeight(); //buffered by wxWidgets
+    return std::max(defaultHeight, fastFromDIP(31)); //default button height is much too small => increase!
+}
+
 
 
 
@@ -47,6 +61,11 @@ void setStandardButtonLayout(wxBoxSizer& sizer, const StdButtons& buttons)
 {
     assert(sizer.GetOrientation() == wxHORIZONTAL);
 
+    //GNOME Human Interface Guidelines: https://developer.gnome.org/hig-book/3.2/hig-book.html#alert-spacing
+    const int spaceH    = fastFromDIP( 6); //OK
+    const int spaceRimH = fastFromDIP(12); //OK
+    const int spaceRimV = fastFromDIP(12); //OK
+
     StdButtons buttonsTmp = buttons;
 
     auto detach = [&](wxButton*& btn)
@@ -54,15 +73,11 @@ void setStandardButtonLayout(wxBoxSizer& sizer, const StdButtons& buttons)
         if (btn)
         {
             assert(btn->GetContainingSizer() == &sizer);
-            if (btn->IsShown())
-            {
-                bool rv = sizer.Detach(btn);
-                assert(rv);
-                if (!rv)
-                    btn = nullptr;
-            }
-            else
-                btn = nullptr;
+            if (btn->IsShown() && sizer.Detach(btn))
+                return;
+
+            assert(false); //why is it hidden!?
+            btn = nullptr;
         }
     };
 
@@ -71,10 +86,30 @@ void setStandardButtonLayout(wxBoxSizer& sizer, const StdButtons& buttons)
     detach(buttonsTmp.btnNo);
     detach(buttonsTmp.btnCancel);
 
-    //GNOME Human Interface Guidelines: https://developer.gnome.org/hig-book/3.2/hig-book.html#alert-spacing
-    const int spaceH    = fastFromDIP( 6); //OK
-    const int spaceRimH = fastFromDIP(12); //OK
-    const int spaceRimV = fastFromDIP(12); //OK
+
+    //"All your fixed-size spacers are belong to us!" => have a clean slate: consider repeated setStandardButtonLayout() calls
+    for (size_t pos = sizer.GetItemCount(); pos-- > 0;)
+        if (wxSizerItem& item = *sizer.GetItem(pos);
+            item.IsSpacer() && item.GetProportion() == 0 && item.GetSize().y == 0)
+        {
+            [[maybe_unused]] const bool rv = sizer.Detach(pos);
+            assert(rv);
+        }
+
+    //set border on left considering existing items
+    if (!sizer.IsEmpty()) //for yet another retarded reason wxWidgets will have wxSizer::GetItem(0) cause an assert rather than just return nullptr as documented
+        if (wxSizerItem& item = *sizer.GetItem(static_cast<size_t>(0));
+            item.IsShown())
+        {
+            assert(item.GetBorder() <= spaceRimV); //pragmatic check: other controls in the sizer should not have a larger border
+
+            if (const int flag = item.GetFlag();
+                flag & wxLEFT)
+                item.SetFlag(flag & ~wxLEFT);
+
+            sizer.Prepend(spaceRimH, 0);
+        }
+
 
     bool settingFirstButton = true;
     auto attach = [&](wxButton* btn)
@@ -82,8 +117,7 @@ void setStandardButtonLayout(wxBoxSizer& sizer, const StdButtons& buttons)
         if (btn)
         {
             assert(btn->GetMinSize().GetHeight() == -1); //let OS or this routine do the sizing! note: OS X does not allow changing the (visible!) button height!
-            const int defaultHeight = wxButton::GetDefaultSize().GetHeight(); //buffered by wxWidgets
-            btn->SetMinSize({-1, std::max(defaultHeight, fastFromDIP(31))}); //default button height is much too small => increase!
+            btn->SetMinSize({-1, getDefaultButtonHeight()});
 
             if (settingFirstButton)
                 settingFirstButton = false;
@@ -93,20 +127,6 @@ void setStandardButtonLayout(wxBoxSizer& sizer, const StdButtons& buttons)
         }
     };
 
-    //set border on left considering existing items
-    if (sizer.GetChildren().GetCount() > 0) //for yet another retarded reason wxWidgets will have wxSizer::GetItem(0) cause an assert rather than just return nullptr as documented
-        if (wxSizerItem* item = sizer.GetItem(static_cast<size_t>(0)))
-        {
-            assert(item->GetBorder() <= spaceRimV); //pragmatic check: other controls in the sizer should not have a larger border
-            int flag = item->GetFlag();
-            if (flag & wxLEFT)
-            {
-                flag &= ~wxLEFT;
-                item->SetFlag(flag);
-            }
-            sizer.Insert(static_cast<size_t>(0), spaceRimH, 0);
-        }
-
     sizer.Add(spaceRimH, 0);
     attach(buttonsTmp.btnNo);
     attach(buttonsTmp.btnCancel);
