@@ -7,10 +7,9 @@
 #ifndef STRING_BASE_H_083217454562342526
 #define STRING_BASE_H_083217454562342526
 
-#include <algorithm>
 #include <atomic>
+#include <utility> //std::exchange
 #include "string_tools.h"
-#include "legacy_compiler.h" //constinit2
 
 
 //Zbase - a policy based string class optimizing performance and flexibility
@@ -93,7 +92,7 @@ protected:
 
     //this needs to be checked before writing to "ptr"
     static bool canWrite(const Char* ptr, size_t minCapacity) { return minCapacity <= descr(ptr)->capacity; }
-    static size_t length(const Char* ptr) { return descr(ptr)->length; }
+    static size_t size(const Char* ptr) { return descr(ptr)->length; }
 
     static void setLength(Char* ptr, size_t newLength)
     {
@@ -175,7 +174,7 @@ protected:
         return d->refCount == 1 && minCapacity <= d->capacity;
     }
 
-    static size_t length(const Char* ptr) { return descr(ptr)->length; }
+    static size_t size(const Char* ptr) { return descr(ptr)->length; }
 
     static void setLength(Char* ptr, size_t newLength)
     {
@@ -209,7 +208,7 @@ private:
     static_assert(offsetof(GlobalEmptyString, nullTerm) - offsetof(GlobalEmptyString, descr) == sizeof(Descriptor), "no gap!");
     static_assert(std::is_trivially_destructible_v<GlobalEmptyString>, "this memory needs to live forever");
 
-    inline static constinit2 GlobalEmptyString globalEmptyString; //constinit: dodge static initialization order fiasco!
+    inline static constinit GlobalEmptyString globalEmptyString; //constinit: dodge static initialization order fiasco!
 };
 
 
@@ -229,17 +228,21 @@ public:
     Zbase();
     Zbase(const Char* str) : Zbase(str, str + strLength(str)) {} //implicit conversion from a C-string!
     Zbase(const Char* str, size_t len) : Zbase(str, str + len) {}
+    explicit Zbase(const std::basic_string_view<Char> view) : Zbase(view.begin(), view.end()) {}
     Zbase(size_t count, Char fillChar);
+    template <class RandomAccessIterator>
+    Zbase(RandomAccessIterator first, RandomAccessIterator last);
     Zbase(const Zbase& str);
     Zbase(Zbase&& tmp) noexcept;
-    template <class InputIterator>
-    Zbase(InputIterator first, InputIterator last);
     //explicit Zbase(Char ch); //dangerous if implicit: Char buffer[]; return buffer[0]; ups... forgot &, but not a compiler error! //-> non-standard extension!!!
 
     ~Zbase();
 
     //operator const Char* () const; //NO implicit conversion to a C-string!! Many problems... one of them: if we forget to provide operator overloads, it'll just work with a Char*...
 
+    operator std::basic_string_view<Char>() const& noexcept { return {data(), size()}; }
+    //operator std::basic_string_view<Char>() const&& = delete; //=> probably a bug!
+
     //STL accessors
     using iterator        = Char*;
     using const_iterator  = const Char*;
@@ -251,31 +254,35 @@ public:
     iterator end  ();
 
     const_iterator begin () const { return rawStr_; }
-    const_iterator end   () const { return rawStr_ + length(); }
+    const_iterator end   () const { return rawStr_ + size(); }
 
     const_iterator cbegin() const { return begin(); }
     const_iterator cend  () const { return end  (); }
 
     //std::string functions
-    size_t length() const;
-    size_t size  () const { return length(); }
+    size_t length() const { return size(); }
+    size_t size  () const;
     const Char* c_str() const { return rawStr_; } //C-string format with 0-termination
+    const Char* data() const { return &*begin(); }
+    /**/  Char* data()       { return &*begin(); }
     const Char& operator[](size_t pos) const;
     /**/  Char& operator[](size_t pos);
-    bool empty() const { return length() == 0; }
+    bool empty() const { return size() == 0; }
     void clear();
+#if 0 //avoid redundant std::string API bloat!
     size_t find (const Zbase& str, size_t pos = 0)    const; //
     size_t find (const Char* str,  size_t pos = 0)    const; //
     size_t find (Char  ch,         size_t pos = 0)    const; //returns "npos" if not found
     size_t rfind(Char  ch,         size_t pos = npos) const; //
     size_t rfind(const Char* str,  size_t pos = npos) const; //
+#endif
     //Zbase& replace(size_t pos1, size_t n1, const Zbase& str);
     void reserve(size_t minCapacity);
     Zbase& assign(const Char* str, size_t len) { return assign(str, str + len); }
     Zbase& append(const Char* str, size_t len) { return append(str, str + len); }
 
-    template <class InputIterator> Zbase& assign(InputIterator first, InputIterator last);
-    template <class InputIterator> Zbase& append(InputIterator first, InputIterator last);
+    template <class RandomAccessIterator> Zbase& assign(RandomAccessIterator first, RandomAccessIterator last);
+    template <class RandomAccessIterator> Zbase& append(RandomAccessIterator first, RandomAccessIterator last);
 
     void resize(size_t newSize, Char fillChar = 0);
     void swap(Zbase& str) { std::swap(rawStr_, str.rawStr_); }
@@ -286,12 +293,17 @@ public:
     Zbase& operator=(const Zbase& str);
     Zbase& operator=(const Char* str)   { return assign(str, strLength(str)); }
     Zbase& operator=(Char ch)           { return assign(&ch, 1); }
-    Zbase& operator+=(const Zbase& str) { return append(str.c_str(), str.length()); }
+    Zbase& operator+=(const Zbase& str) { return append(str.c_str(), str.size()); }
     Zbase& operator+=(const Char* str)  { return append(str, strLength(str)); }
     Zbase& operator+=(Char ch)          { return append(&ch, 1); }
+    Zbase& operator+=(const std::basic_string_view<Char> str) { return append(str.begin(), str.end()); }
 
     static const size_t npos = static_cast<size_t>(-1);
 
+    inline friend Zbase operator+(                       const Char* lhs, const Zbase& rhs) { return Zbase(lhs, strLength(lhs),    rhs.c_str(), rhs.size()); }
+    inline friend Zbase operator+(                             Char  lhs, const Zbase& rhs) { return Zbase(&lhs, 1,                rhs.c_str(), rhs.size()); }
+    inline friend Zbase operator+(const std::basic_string_view<Char> lhs, const Zbase& rhs) { return Zbase(lhs.data(), lhs.size(), rhs.c_str(), rhs.size()); }
+
 private:
     Zbase              (int) = delete; //
     Zbase(size_t count, int) = delete; //
@@ -299,6 +311,16 @@ private:
     Zbase& operator+=  (int) = delete; //
     void   push_back   (int) = delete; //
 
+    Zbase              (std::nullptr_t) = delete;
+    Zbase(size_t count, std::nullptr_t) = delete;
+    Zbase& operator=   (std::nullptr_t) = delete;
+    Zbase& operator+=  (std::nullptr_t) = delete;
+    void   push_back   (std::nullptr_t) = delete;
+
+    //not part of std::string API => private:
+    Zbase(const Char* str1, size_t len1, const Char* str2, size_t len2);
+    //alternative: Zbase() + reserve() + 2 x append()
+
     Char* rawStr_;
 };
 
@@ -308,21 +330,21 @@ template <class Char, template <class> class SP>        bool operator==(const Zb
 template <class Char, template <class> class SP>        bool operator==(const Zbase<Char, SP>& lhs, const Char*            rhs);
 template <class Char, template <class> class SP> inline bool operator==(const Char*            lhs, const Zbase<Char, SP>& rhs) { return operator==(rhs, lhs); }
 
-template <class Char, template <class> class SP> std::strong_ordering operator<=>(const Zbase<Char, SP>& lhs, const Zbase<Char, SP>& rhs);
-template <class Char, template <class> class SP> std::strong_ordering operator<=>(const Zbase<Char, SP>& lhs, const Char*            rhs);
-template <class Char, template <class> class SP> std::strong_ordering operator<=>(const Char*            lhs, const Zbase<Char, SP>& rhs);
+//follow convention + compare by unsigned char; alternative: std::lexicographical_compare_three_way + reinterpret_cast<const std::make_unsigned_t<Char>*>()
+template <class Char, template <class> class SP> std::strong_ordering operator<=>(const Zbase<Char, SP>& lhs, const Zbase<Char, SP>& rhs) { return compareString(lhs, rhs); }
+template <class Char, template <class> class SP> std::strong_ordering operator<=>(const Zbase<Char, SP>& lhs, const Char*            rhs) { return compareString(lhs, rhs); }
+template <class Char, template <class> class SP> std::strong_ordering operator<=>(const Char*            lhs, const Zbase<Char, SP>& rhs) { return compareString(lhs, rhs); }
 
 template <class Char, template <class> class SP> inline Zbase<Char, SP> operator+(const Zbase<Char, SP>& lhs, const Zbase<Char, SP>& rhs) { return Zbase<Char, SP>(lhs) += rhs; }
 template <class Char, template <class> class SP> inline Zbase<Char, SP> operator+(const Zbase<Char, SP>& lhs, const Char*            rhs) { return Zbase<Char, SP>(lhs) += rhs; }
 template <class Char, template <class> class SP> inline Zbase<Char, SP> operator+(const Zbase<Char, SP>& lhs,       Char             rhs) { return Zbase<Char, SP>(lhs) += rhs; }
+template <class Char, template <class> class SP> inline Zbase<Char, SP> operator+(const Zbase<Char, SP>& lhs, const std::basic_string_view<Char> rhs) { return Zbase<Char, SP>(lhs) += rhs; }
 
 //don't use unified first argument but save one move-construction in the r-value case instead!
 template <class Char, template <class> class SP> inline Zbase<Char, SP> operator+(Zbase<Char, SP>&& lhs, const Zbase<Char, SP>& rhs) { return std::move(lhs += rhs); } //the move *is* needed!!!
 template <class Char, template <class> class SP> inline Zbase<Char, SP> operator+(Zbase<Char, SP>&& lhs, const Char*            rhs) { return std::move(lhs += rhs); } //lhs, is an l-value parameter...
 template <class Char, template <class> class SP> inline Zbase<Char, SP> operator+(Zbase<Char, SP>&& lhs,       Char             rhs) { return std::move(lhs += rhs); } //and not a local variable => no copy elision
-
-template <class Char, template <class> class SP> inline Zbase<Char, SP> operator+(const Char*       lhs, const Zbase<Char, SP>& rhs) { return Zbase<Char, SP>(lhs    ) += rhs; }
-template <class Char, template <class> class SP> inline Zbase<Char, SP> operator+(      Char        lhs, const Zbase<Char, SP>& rhs) { return Zbase<Char, SP>(&lhs, 1) += rhs; }
+template <class Char, template <class> class SP> inline Zbase<Char, SP> operator+(Zbase<Char, SP>&& lhs, const std::basic_string_view<Char> rhs) { return std::move(lhs += rhs); }
 
 template <class Char, template <class> class SP> inline Zbase<Char, SP> operator+(const Zbase<Char, SP>&, int) = delete; //detect usage errors
 template <class Char, template <class> class SP> inline Zbase<Char, SP> operator+(int, const Zbase<Char, SP>&) = delete; //
@@ -346,10 +368,10 @@ Zbase<Char, SP>::Zbase()
 
 
 template <class Char, template <class> class SP>
-template <class InputIterator> inline
-Zbase<Char, SP>::Zbase(InputIterator first, InputIterator last)
+template <class RandomAccessIterator> inline
+Zbase<Char, SP>::Zbase(RandomAccessIterator first, RandomAccessIterator last)
 {
-    rawStr_ = this->create(std::distance(first, last));
+    rawStr_ = this->create(last - first);
     *std::copy(first, last, rawStr_) = 0;
 }
 
@@ -373,12 +395,21 @@ Zbase<Char, SP>::Zbase(const Zbase<Char, SP>& str)
 template <class Char, template <class> class SP> inline
 Zbase<Char, SP>::Zbase(Zbase<Char, SP>&& tmp) noexcept
 {
-    rawStr_ = tmp.rawStr_;
-    tmp.rawStr_ = nullptr; //usually nullptr would violate the class invarants, but it is good enough for the destructor!
+    rawStr_ = std::exchange(tmp.rawStr_, nullptr);
+    //usually nullptr would violate the class invarants, but it is good enough for the destructor!
     //caveat: do not increment ref-count of an unshared string! We'd lose optimization opportunity of reusing its memory!
 }
 
 
+template <class Char, template <class> class SP> inline
+Zbase<Char, SP>::Zbase(const Char* str1, size_t len1, const Char* str2, size_t len2)
+{
+    rawStr_ = this->create(len1 + len2);
+    std::copy (str1, str1 + len1, rawStr_);
+    *std::copy(str2, str2 + len2, rawStr_ + len1) = 0;
+}
+
+
 template <class Char, template <class> class SP> inline
 Zbase<Char, SP>::~Zbase()
 {
@@ -388,35 +419,36 @@ Zbase<Char, SP>::~Zbase()
 }
 
 
+#if 0 //avoid redundant std::string API bloat!
 template <class Char, template <class> class SP> inline
-size_t Zbase<Char, SP>::find(const Zbase& str, size_t pos) const
+size_t Zbase<Char, SP>::find(const Zbase& str, size_t pos) const //returns "npos" if not found
 {
-    assert(pos <= length());
-    const size_t len = length();
+    assert(pos <= size());
+    const size_t len = size();
     const Char* thisEnd = begin() + len; //respect embedded 0
-    const Char* it = std::search(begin() + std::min(pos, len), thisEnd,
+    const Char* it = searchFirst(begin() + std::min(pos, len), thisEnd,
                                  str.begin(), str.end());
     return it == thisEnd ? npos : it - begin();
 }
 
 
 template <class Char, template <class> class SP> inline
-size_t Zbase<Char, SP>::find(const Char* str, size_t pos) const
+size_t Zbase<Char, SP>::find(const Char* str, size_t pos) const //returns "npos" if not found
 {
-    assert(pos <= length());
-    const size_t len = length();
+    assert(pos <= size());
+    const size_t len = size();
     const Char* thisEnd = begin() + len; //respect embedded 0
-    const Char* it = std::search(begin() + std::min(pos, len), thisEnd,
+    const Char* it = searchFirst(begin() + std::min(pos, len), thisEnd,
                                  str, str + strLength(str));
     return it == thisEnd ? npos : it - begin();
 }
 
 
 template <class Char, template <class> class SP> inline
-size_t Zbase<Char, SP>::find(Char ch, size_t pos) const
+size_t Zbase<Char, SP>::find(Char ch, size_t pos) const //returns "npos" if not found
 {
-    assert(pos <= length());
-    const size_t len = length();
+    assert(pos <= size());
+    const size_t len = size();
     const Char* thisEnd = begin() + len; //respect embedded 0
     const Char* it = std::find(begin() + std::min(pos, len), thisEnd, ch);
     return it == thisEnd ? npos : it - begin();
@@ -424,10 +456,10 @@ size_t Zbase<Char, SP>::find(Char ch, size_t pos) const
 
 
 template <class Char, template <class> class SP> inline
-size_t Zbase<Char, SP>::rfind(Char ch, size_t pos) const
+size_t Zbase<Char, SP>::rfind(Char ch, size_t pos) const //returns "npos" if not found
 {
-    assert(pos == npos || pos <= length());
-    const size_t len = length();
+    assert(pos == npos || pos <= size());
+    const size_t len = size();
     const Char* currEnd = begin() + (pos == npos ? len : std::min(pos + 1, len));
     const Char* it = findLast(begin(), currEnd, ch);
     return it == currEnd ? npos : it - begin();
@@ -435,22 +467,23 @@ size_t Zbase<Char, SP>::rfind(Char ch, size_t pos) const
 
 
 template <class Char, template <class> class SP> inline
-size_t Zbase<Char, SP>::rfind(const Char* str, size_t pos) const
+size_t Zbase<Char, SP>::rfind(const Char* str, size_t pos) const //returns "npos" if not found
 {
-    assert(pos == npos || pos <= length());
+    assert(pos == npos || pos <= size());
     const size_t strLen = strLength(str);
-    const size_t len = length();
+    const size_t len = size();
     const Char* currEnd = begin() + (pos == npos ? len : std::min(pos + strLen, len));
     const Char* it = searchLast(begin(), currEnd,
                                 str, str + strLen);
     return it == currEnd ? npos : it - begin();
 }
+#endif
 
 
 template <class Char, template <class> class SP> inline
 void Zbase<Char, SP>::resize(size_t newSize, Char fillChar)
 {
-    const size_t oldSize = length();
+    const size_t oldSize = size();
     if (this->canWrite(rawStr_, newSize))
     {
         if (oldSize < newSize)
@@ -479,52 +512,28 @@ void Zbase<Char, SP>::resize(size_t newSize, Char fillChar)
 template <class Char, template <class> class SP> inline
 bool operator==(const Zbase<Char, SP>& lhs, const Zbase<Char, SP>& rhs)
 {
-    return lhs.length() == rhs.length() && std::equal(lhs.begin(), lhs.end(), rhs.begin()); //respect embedded 0
+    return lhs.size() == rhs.size() && std::equal(lhs.begin(), lhs.end(), rhs.begin()); //respect embedded 0
 }
 
 
 template <class Char, template <class> class SP> inline
 bool operator==(const Zbase<Char, SP>& lhs, const Char* rhs)
 {
-    return lhs.length() == strLength(rhs) && std::equal(lhs.begin(), lhs.end(), rhs); //respect embedded 0
+    return lhs.size() == strLength(rhs) && std::equal(lhs.begin(), lhs.end(), rhs); //respect embedded 0
 }
 
 
 template <class Char, template <class> class SP> inline
-std::strong_ordering operator<=>(const Zbase<Char, SP>& lhs, const Zbase<Char, SP>& rhs)
+size_t Zbase<Char, SP>::size() const
 {
-    return std::lexicographical_compare_three_way(lhs.begin(), lhs.end(),  //respect embedded 0
-                                                  rhs.begin(), rhs.end()); //
-}
-
-
-template <class Char, template <class> class SP> inline
-std::strong_ordering operator<=>(const Zbase<Char, SP>& lhs, const Char* rhs)
-{
-    return std::lexicographical_compare_three_way(lhs.begin(), lhs.end(), //respect embedded 0
-                                                  rhs, rhs + strLength(rhs));
-}
-
-
-template <class Char, template <class> class SP> inline
-std::strong_ordering operator<=>(const Char* lhs, const Zbase<Char, SP>& rhs)
-{
-    return std::lexicographical_compare_three_way(lhs, lhs + strLength(lhs),
-                                                  rhs.begin(), rhs.end()); //respect embedded 0
-}
-
-
-template <class Char, template <class> class SP> inline
-size_t Zbase<Char, SP>::length() const
-{
-    return SP<Char>::length(rawStr_);
+    return SP<Char>::size(rawStr_);
 }
 
 
 template <class Char, template <class> class SP> inline
 const Char& Zbase<Char, SP>::operator[](size_t pos) const
 {
-    assert(pos < length()); //design by contract! no runtime check!
+    assert(pos < size()); //design by contract! no runtime check!
     return rawStr_[pos];
 }
 
@@ -532,8 +541,8 @@ const Char& Zbase<Char, SP>::operator[](size_t pos) const
 template <class Char, template <class> class SP> inline
 Char& Zbase<Char, SP>::operator[](size_t pos)
 {
-    reserve(length()); //make unshared!
-    assert(pos < length()); //design by contract! no runtime check!
+    reserve(size()); //make unshared!
+    assert(pos < size()); //design by contract! no runtime check!
     return rawStr_[pos];
 }
 
@@ -541,7 +550,7 @@ Char& Zbase<Char, SP>::operator[](size_t pos)
 template <class Char, template <class> class SP> inline
 auto Zbase<Char, SP>::begin() -> iterator
 {
-    reserve(length()); //make unshared!
+    reserve(size()); //make unshared!
     return rawStr_;
 }
 
@@ -549,7 +558,7 @@ auto Zbase<Char, SP>::begin() -> iterator
 template <class Char, template <class> class SP> inline
 auto Zbase<Char, SP>::end() -> iterator
 {
-    return begin() + length();
+    return begin() + size();
 }
 
 
@@ -575,9 +584,9 @@ void Zbase<Char, SP>::reserve(size_t minCapacity) //make unshared and check capa
     if (!this->canWrite(rawStr_, minCapacity))
     {
         //allocate a new string
-        const size_t len = length();
+        const size_t len = size();
         Char* newStr = this->create(len, std::max(len, minCapacity)); //reserve() must NEVER shrink the string: logical const!
-        std::copy(rawStr_, rawStr_ + len + 1, newStr); //include 0-termination
+        *std::copy(rawStr_, rawStr_ + len, newStr) = 0;
 
         this->destroy(rawStr_);
         rawStr_ = newStr;
@@ -586,10 +595,10 @@ void Zbase<Char, SP>::reserve(size_t minCapacity) //make unshared and check capa
 
 
 template <class Char, template <class> class SP>
-template <class InputIterator> inline
-Zbase<Char, SP>& Zbase<Char, SP>::assign(InputIterator first, InputIterator last)
+template <class RandomAccessIterator> inline
+Zbase<Char, SP>& Zbase<Char, SP>::assign(RandomAccessIterator first, RandomAccessIterator last)
 {
-    const size_t len = std::distance(first, last);
+    const size_t len = last - first;
     if (this->canWrite(rawStr_, len))
     {
         *std::copy(first, last, rawStr_) = 0;
@@ -603,13 +612,13 @@ Zbase<Char, SP>& Zbase<Char, SP>::assign(InputIterator first, InputIterator last
 
 
 template <class Char, template <class> class SP>
-template <class InputIterator> inline
-Zbase<Char, SP>& Zbase<Char, SP>::append(InputIterator first, InputIterator last)
+template <class RandomAccessIterator> inline
+Zbase<Char, SP>& Zbase<Char, SP>::append(RandomAccessIterator first, RandomAccessIterator last)
 {
-    const size_t len = std::distance(first, last);
+    const size_t len = last - first; //std::distance(first, last);
     if (len > 0) //avoid making this string unshared for no reason
     {
-        const size_t thisLen = length();
+        const size_t thisLen = size();
         reserve(thisLen + len); //make unshared and check capacity
 
         *std::copy(first, last, rawStr_ + thisLen) = 0;
@@ -631,10 +640,10 @@ Zbase<Char, SP>& Zbase<Char, SP>::operator=(const Zbase<Char, SP>& str)
 template <class Char, template <class> class SP> inline
 Zbase<Char, SP>& Zbase<Char, SP>::operator=(Zbase<Char, SP>&& tmp) noexcept
 {
-    //don't use swap() but end rawStr_ life time immediately
+    //don't swap() but end rawStr_ life time immediately
     this->destroy(rawStr_);
-    rawStr_ = tmp.rawStr_;
-    tmp.rawStr_ = nullptr;
+
+    rawStr_ = std::exchange(tmp.rawStr_, nullptr);
     return *this;
 }
 
@@ -642,11 +651,19 @@ Zbase<Char, SP>& Zbase<Char, SP>::operator=(Zbase<Char, SP>&& tmp) noexcept
 template <class Char, template <class> class SP> inline
 void Zbase<Char, SP>::pop_back()
 {
-    const size_t len = length();
+    const size_t len = size();
     assert(len > 0);
     if (len > 0)
         resize(len - 1);
 }
 }
 
+
+//std::hash specialization in global namespace
+template <class Char, template <class> class SP>
+struct std::hash<zen::Zbase<Char, SP>>
+{
+    size_t operator()(const zen::Zbase<Char, SP>& str) const { return zen::hashString<size_t>(str); }
+};
+
 #endif //STRING_BASE_H_083217454562342526
