@@ -15,6 +15,8 @@ namespace zen
 //COM needs to be initialized before calling any of these functions! CoInitializeEx/CoUninitialize
 
 Zstring getLoginUser(); //throw FileError
+Zstring getUserDescription();//throw FileError
+
 
 struct ComputerModel
 {
@@ -33,6 +35,7 @@ Zstring getRealProcessPath(); //throw FileError
 Zstring getUserDownloadsPath(); //throw FileError
 Zstring getUserDataPath(); //throw FileError
 
+Zstring getUserHome(); //throw FileError
 }
 
 #endif //SYSTEM_H_4189731847832147508915
