@@ -9,9 +9,7 @@
 
 #include <thread>
 #include <future>
-#include "scope_guard.h"
 #include "ring_buffer.h"
-#include "string_tools.h"
 #include "zstring.h"
 
 
@@ -65,7 +63,7 @@ class ThreadStopRequest {};
 //context of worker thread:
 void interruptionPoint(); //throw ThreadStopRequest
 
-template<class Predicate>
+template <class Predicate>
 void interruptibleWait(std::condition_variable& cv, std::unique_lock<std::mutex>& lock, Predicate pred); //throw ThreadStopRequest
 
 template <class Rep, class Period>
@@ -74,6 +72,7 @@ void interruptibleSleep(const std::chrono::duration<Rep, Period>& relTime); //th
 void setCurrentThreadName(const Zstring& threadName);
 
 bool runningOnMainThread();
+
 //------------------------------------------------------------------------------------------
 
 /*  std::async replacement without crappy semantics:
@@ -90,7 +89,7 @@ auto runAsync(Function&& fun);
 
 //wait for all with a time limit: return true if *all* results are available!
 //TODO: use std::when_all when available
-template<class InputIterator, class Duration>
+template <class InputIterator, class Duration>
 bool waitForAllTimed(InputIterator first, InputIterator last, const Duration& wait_duration);
 
 template<typename T> inline
@@ -123,7 +122,7 @@ private:
 //------------------------------------------------------------------------------------------
 
 //value associated with mutex and guaranteed protected access:
-//TODO: use std::synchronized_value when available http://www.open-std.org/jtc1/sc22/wg21/docs/papers/2017/p0290r2.html
+//TODO: use std::synchronized_value when available https://isocpp.github.io/CppCoreGuidelines/CppCoreGuidelines#Rconc-mutex
 template <class T>
 class Protected
 {
@@ -162,7 +161,7 @@ public:
     ~ThreadGroup()
     {
         for (InterruptibleThread& w : worker_)
-            w.requestStop(); //stop *all* at the same time before join!
+            w.requestStop(); //similar, but not the same as ~InterruptibleThread: stop *all* at the same time before join!
 
         if (detach_) //detach() without requestStop() doesn't make sense
             for (InterruptibleThread& w : worker_)
@@ -191,13 +190,13 @@ public:
     void wait()
     {
         //perf: no difference in xBRZ test case compared to std::condition_variable-based implementation
-        auto promiseDone = std::make_shared<std::promise<void>>(); //
-        std::future<void> allDone = promiseDone->get_future();
+        auto promDone = std::make_shared<std::promise<void>>(); //
+        std::future<void> futDone = promDone->get_future();
 
-        notifyWhenDone([promiseDone] { promiseDone->set_value(); }); //std::function doesn't support construction involving move-only types!
+        notifyWhenDone([promDone] { promDone->set_value(); }); //std::function doesn't support construction involving move-only types!
         //use reference? => potential lifetime issue, e.g. promise object theoretically might be accessed inside set_value() after future gets signalled
 
-        allDone.get();
+        futDone.get();
     }
 
     //non-blocking wait()-alternative: context of controlling thread:
@@ -311,7 +310,7 @@ auto runAsync(Function&& fun)
 }
 
 
-template<class InputIterator, class Duration> inline
+template <class InputIterator, class Duration> inline
 bool waitForAllTimed(InputIterator first, InputIterator last, const Duration& duration)
 {
     const std::chrono::steady_clock::time_point stopTime = std::chrono::steady_clock::now() + duration;
@@ -417,7 +416,7 @@ public:
     }
 
     //context of worker thread:
-    template<class Predicate>
+    template <class Predicate>
     void interruptibleWait(std::condition_variable& cv, std::unique_lock<std::mutex>& lock, Predicate pred) //throw ThreadStopRequest
     {
         setConditionVar(&cv);
@@ -447,7 +446,7 @@ private:
         activeCondition_ = cv;
     }
 
-    std::atomic<bool> stopRequested_{false}; //std:atomic is uninitialized by default!!!
+    std::atomic<bool> stopRequested_{false}; //std::atomic is uninitialized by default!!!
     //"The default constructor is trivial: no initialization takes place other than zero initialization of static and thread-local objects."
 
     std::condition_variable* activeCondition_ = nullptr;
@@ -476,7 +475,7 @@ void interruptionPoint() //throw ThreadStopRequest
 
 
 //context of worker thread:
-template<class Predicate> inline
+template <class Predicate> inline
 void interruptibleWait(std::condition_variable& cv, std::unique_lock<std::mutex>& lock, Predicate pred) //throw ThreadStopRequest
 {
     assert(impl::threadLocalInterruptionStatus);
